@@ -1,11 +1,11 @@
 /*
- * Copyright 2016-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,6 +13,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #pragma once
 
 #include <fstream>
@@ -21,7 +22,7 @@
 #include <stdexcept>
 
 #include <boost/filesystem.hpp>
-#include <thrift/compiler/mustache/mstch.h>
+#include <thrift/compiler/detail/mustache/mstch.h>
 
 #include <thrift/compiler/generate/t_generator.h>
 #include <thrift/compiler/generate/t_mstch_objects.h>
@@ -39,6 +40,8 @@ class t_mstch_generator : public t_generator {
       std::map<std::string, std::string> parsed_options,
       bool convert_delimiter = false);
 
+  virtual ~t_mstch_generator() override;
+
  protected:
   /**
    * Option pairs specified on command line for influencing generation behavior
@@ -49,9 +52,7 @@ class t_mstch_generator : public t_generator {
    * If true, typedefs will be automatically resolved to their underlying
    * type.
    */
-  virtual bool should_resolve_typedefs() const {
-    return false;
-  }
+  virtual bool should_resolve_typedefs() const { return false; }
 
   /**
    * Fetches a particular template from the template map, throwing an error
@@ -71,16 +72,14 @@ class t_mstch_generator : public t_generator {
    * Render the mstch template with name `template_name` in the given context.
    */
   std::string render(
-      const std::string& template_name,
-      const mstch::node& context);
+      const std::string& template_name, const mstch::node& context);
 
   /**
    * Write an output file with the given contents to a path
    * under the output directory.
    */
   void write_output(
-      const boost::filesystem::path& path,
-      const std::string& data);
+      const boost::filesystem::path& path, const std::string& data);
 
   /**
    * Render the mstch template with name `template_name` in the given context
@@ -149,7 +148,7 @@ class t_mstch_generator : public t_generator {
   mstch::map dump(const t_const_value&);
   mstch::map dump(const std::map<t_const_value*, t_const_value*>::value_type&);
 
-  using annotation = std::pair<std::string, std::string>;
+  using annotation = std::pair<std::string, annotation_value>;
   mstch::map dump(const annotation&);
   mstch::map dump(const std::string&);
 
@@ -205,8 +204,8 @@ class t_mstch_generator : public t_generator {
     }
   }
 
-  bool has_option(const std::string& key);
-  std::unique_ptr<std::string> get_option(const std::string& key);
+  bool has_option(const std::string& option) const;
+  std::string get_option(const std::string& option);
 
  private:
   std::map<std::string, std::string> template_map_;
@@ -232,6 +231,8 @@ class t_mstch_generator : public t_generator {
  protected:
   std::shared_ptr<mstch_generators> generators_;
   std::shared_ptr<mstch_cache> cache_;
+
+  const std::shared_ptr<mstch_base>& cached_program(t_program const* program);
 };
 
 } // namespace compiler
