@@ -1,11 +1,11 @@
 /*
- * Copyright 2017-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,8 +16,8 @@
 
 #pragma once
 
-#include <folly/Synchronized.h>
 #include <vector>
+#include <folly/Synchronized.h>
 
 #include <thrift/example/if/gen-cpp2/ChatRoomService.h>
 
@@ -44,5 +44,5 @@ class ChatRoomServiceHandler : virtual public ChatRoomServiceSvIf {
   folly::Synchronized<std::vector<Message>> messageBuffer_ =
       folly::Synchronized<std::vector<Message>>();
 };
-} // chatroom
-} // example
+} // namespace chatroom
+} // namespace example
