@@ -1,11 +1,11 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,26 +13,39 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #pragma once
 
 #include <chrono>
 #include <map>
+#include <optional>
 #include <string>
 
+#include <folly/Try.h>
 #include <folly/io/IOBuf.h>
 
 #include <thrift/lib/cpp/ContextStack.h>
 #include <thrift/lib/cpp/concurrency/Thread.h>
 #include <thrift/lib/cpp/transport/THeader.h>
-#include <thrift/lib/cpp2/async/ClientStreamBridge.h>
-#ifdef FOLLY_HAS_COROUTINES
 #include <thrift/lib/cpp2/async/ClientSinkBridge.h>
-#endif
-#include <thrift/lib/cpp2/async/SemiStream.h>
-#include <thrift/lib/cpp2/async/Stream.h>
+#include <thrift/lib/cpp2/async/ClientStreamBridge.h>
+#include <thrift/lib/cpp2/async/Interaction.h>
+#include <thrift/lib/cpp2/async/RpcOptions.h>
+#include <thrift/lib/cpp2/async/RpcTypes.h>
+#include <thrift/lib/cpp2/protocol/Protocol.h>
 
 namespace apache {
 namespace thrift {
+
+struct RpcSizeStats {
+  RpcSizeStats() = default;
+
+  uint32_t requestSerializedSizeBytes{0};
+  uint32_t requestWireSizeBytes{0};
+  uint32_t responseSerializedSizeBytes{0};
+  uint32_t responseWireSizeBytes{0};
+};
+
 class ClientReceiveState {
  public:
   ClientReceiveState() : protocolId_(-1) {}
@@ -41,102 +54,101 @@ class ClientReceiveState {
       uint16_t _protocolId,
       std::unique_ptr<folly::IOBuf> _buf,
       std::unique_ptr<apache::thrift::transport::THeader> _header,
-      std::shared_ptr<apache::thrift::ContextStack> _ctx)
+      std::unique_ptr<apache::thrift::ContextStack> _ctx)
       : protocolId_(_protocolId),
         ctx_(std::move(_ctx)),
-        buf_(std::move(_buf)),
-        header_(std::move(_header)) {}
+        header_(std::move(_header)) {
+    initFromLegacyFormat(std::move(_buf));
+  }
   ClientReceiveState(
       uint16_t _protocolId,
-      ResponseAndSemiStream<
-          std::unique_ptr<folly::IOBuf>,
-          std::unique_ptr<folly::IOBuf>> bufAndStream,
+      std::unique_ptr<folly::IOBuf> _buf,
       std::unique_ptr<apache::thrift::transport::THeader> _header,
-      std::shared_ptr<apache::thrift::ContextStack> _ctx)
+      std::unique_ptr<apache::thrift::ContextStack> _ctx,
+      const RpcSizeStats& _rpcWireSizeStats)
       : protocolId_(_protocolId),
         ctx_(std::move(_ctx)),
-        buf_(std::move(bufAndStream.response)),
         header_(std::move(_header)),
-        stream_(std::move(bufAndStream.stream)) {}
+        rpcSizeStats_(_rpcWireSizeStats) {
+    initFromLegacyFormat(std::move(_buf));
+  }
   ClientReceiveState(
       uint16_t _protocolId,
       std::unique_ptr<folly::IOBuf> buf,
       std::unique_ptr<apache::thrift::transport::THeader> _header,
-      std::shared_ptr<apache::thrift::ContextStack> _ctx,
-      detail::ClientStreamBridge::Ptr streamBridge,
-      int32_t chunkBufferSize)
+      std::unique_ptr<apache::thrift::ContextStack> _ctx,
+      apache::thrift::detail::ClientStreamBridge::ClientPtr streamBridge,
+      const BufferOptions& bufferOptions)
       : protocolId_(_protocolId),
         ctx_(std::move(_ctx)),
-        buf_(std::move(buf)),
         header_(std::move(_header)),
         streamBridge_(std::move(streamBridge)),
-        chunkBufferSize_(std::move(chunkBufferSize)) {}
+        bufferOptions_(bufferOptions) {
+    initFromLegacyFormat(std::move(buf));
+  }
   ClientReceiveState(
       folly::exception_wrapper _excw,
-      std::shared_ptr<apache::thrift::ContextStack> _ctx)
+      std::unique_ptr<apache::thrift::ContextStack> _ctx)
       : protocolId_(-1),
         ctx_(std::move(_ctx)),
         header_(std::make_unique<apache::thrift::transport::THeader>()),
         excw_(std::move(_excw)) {}
-#ifdef FOLLY_HAS_COROUTINES
   ClientReceiveState(
       uint16_t _protocolId,
       std::unique_ptr<folly::IOBuf> _buf,
-      detail::ClientSinkBridge::Ptr sink,
+      apache::thrift::detail::ClientSinkBridge::Ptr sink,
       std::unique_ptr<apache::thrift::transport::THeader> _header,
-      std::shared_ptr<apache::thrift::ContextStack> _ctx)
+      std::unique_ptr<apache::thrift::ContextStack> _ctx)
       : protocolId_(_protocolId),
         ctx_(std::move(_ctx)),
-        buf_(std::move(_buf)),
         header_(std::move(_header)),
-        sink_(std::move(sink)) {}
-#endif
+        sink_(std::move(sink)) {
+    initFromLegacyFormat(std::move(_buf));
+  }
+  ClientReceiveState(
+      uint16_t _protocolId,
+      MessageType mtype,
+      SerializedResponse response,
+      std::unique_ptr<apache::thrift::transport::THeader> _header,
+      std::unique_ptr<apache::thrift::ContextStack> _ctx,
+      const RpcSizeStats& _rpcWireSizeStats)
+      : protocolId_(_protocolId),
+        messageType_(mtype),
+        ctx_(std::move(_ctx)),
+        response_(std::move(response)),
+        header_(std::move(_header)),
+        rpcSizeStats_(_rpcWireSizeStats) {}
 
-  bool isException() const {
-    return excw_ ? true : false;
+  bool isException() const { return excw_ ? true : false; }
+
+  folly::exception_wrapper const& exception() const { return excw_; }
+
+  folly::exception_wrapper& exception() { return excw_; }
+
+  uint16_t protocolId() const { return protocolId_; }
+
+  bool hasResponseBuffer() const { return (bool)response_.buffer; }
+
+  MessageType messageType() const { return messageType_; }
+
+  SerializedResponse& serializedResponse() { return response_; }
+  const SerializedResponse& serializedResponse() const { return response_; }
+
+  SerializedResponse extractSerializedResponse() {
+    return std::move(response_);
   }
 
-  folly::exception_wrapper const& exception() const {
-    return excw_;
-  }
-
-  folly::exception_wrapper& exception() {
-    return excw_;
-  }
-
-  uint16_t protocolId() const {
-    return protocolId_;
-  }
-
-  folly::IOBuf* buf() const {
-    return buf_.get();
-  }
-
-  std::unique_ptr<folly::IOBuf> extractBuf() {
-    return std::move(buf_);
-  }
-
-  SemiStream<std::unique_ptr<folly::IOBuf>> extractStream() {
-    return std::move(stream_);
-  }
-
-  detail::ClientStreamBridge::Ptr extractStreamBridge() {
+  apache::thrift::detail::ClientStreamBridge::ClientPtr extractStreamBridge() {
     return std::move(streamBridge_);
   }
 
-#ifdef FOLLY_HAS_COROUTINES
-  detail::ClientSinkBridge::Ptr extractSink() {
+  apache::thrift::detail::ClientSinkBridge::Ptr extractSink() {
     return std::move(sink_);
   }
-#endif
 
-  int32_t chunkBufferSize() const {
-    return chunkBufferSize_;
-  }
+  const BufferOptions& bufferOptions() const { return bufferOptions_; }
 
-  apache::thrift::transport::THeader* header() const {
-    return header_.get();
-  }
+  apache::thrift::transport::THeader* header() const { return header_.get(); }
 
   std::unique_ptr<apache::thrift::transport::THeader> extractHeader() {
     return std::move(header_);
@@ -146,30 +158,58 @@ class ClientReceiveState {
     header_ = std::move(h);
   }
 
-  apache::thrift::ContextStack* ctx() const {
-    return ctx_.get();
-  }
+  apache::thrift::ContextStack* ctx() const { return ctx_.get(); }
 
-  void resetProtocolId(uint16_t protocolId) {
-    protocolId_ = protocolId;
-  }
+  void resetProtocolId(uint16_t protocolId) { protocolId_ = protocolId; }
 
-  void resetCtx(std::shared_ptr<apache::thrift::ContextStack> _ctx) {
+  void resetCtx(std::unique_ptr<apache::thrift::ContextStack> _ctx) {
     ctx_ = std::move(_ctx);
   }
 
+  const RpcSizeStats& getRpcSizeStats() const { return rpcSizeStats_; }
+
+  static ClientReceiveState create(
+      std::unique_ptr<folly::IOBuf> buf,
+      std::unique_ptr<apache::thrift::transport::THeader> tHeader,
+      apache::thrift::detail::ClientStreamBridge::ClientPtr clientStreamBridge,
+      BufferOptions bufferOptions = BufferOptions(),
+      uint16_t protocolId = static_cast<uint16_t>(-1)) {
+    return ClientReceiveState(
+        protocolId,
+        std::move(buf),
+        std::move(tHeader),
+        nullptr,
+        std::move(clientStreamBridge),
+        bufferOptions);
+  }
+
+  static ClientReceiveState create(
+      std::unique_ptr<folly::IOBuf> buf,
+      std::unique_ptr<apache::thrift::transport::THeader> tHeader,
+      apache::thrift::detail::ClientSinkBridge::Ptr clientSinkBridge,
+      BufferOptions,
+      uint16_t protocolId = static_cast<uint16_t>(-1)) {
+    return ClientReceiveState(
+        protocolId,
+        std::move(buf),
+        std::move(clientSinkBridge),
+        std::move(tHeader),
+        nullptr);
+  }
+
  private:
   uint16_t protocolId_;
-  std::shared_ptr<apache::thrift::ContextStack> ctx_;
-  std::unique_ptr<folly::IOBuf> buf_;
+  MessageType messageType_{MessageType::T_CALL};
+  std::unique_ptr<apache::thrift::ContextStack> ctx_;
+  SerializedResponse response_{nullptr};
   std::unique_ptr<apache::thrift::transport::THeader> header_;
   folly::exception_wrapper excw_;
-  SemiStream<std::unique_ptr<folly::IOBuf>> stream_;
-  detail::ClientStreamBridge::Ptr streamBridge_;
-#ifdef FOLLY_HAS_COROUTINES
-  detail::ClientSinkBridge::Ptr sink_;
-#endif
-  int32_t chunkBufferSize_;
+  apache::thrift::detail::ClientStreamBridge::ClientPtr streamBridge_;
+  apache::thrift::detail::ClientSinkBridge::Ptr sink_;
+  BufferOptions bufferOptions_;
+  RpcSizeStats rpcSizeStats_;
+
+  void initFromLegacyFormat(std::unique_ptr<folly::IOBuf> buffer);
 };
 
 class RequestClientCallback {
@@ -184,21 +224,17 @@ class RequestClientCallback {
       std::unique_ptr<RequestClientCallback, RequestClientCallbackDeleter>;
 
   virtual ~RequestClientCallback() {}
-  virtual void onRequestSent() noexcept = 0;
+  virtual void onRequestSent() noexcept {}
   virtual void onResponse(ClientReceiveState&&) noexcept = 0;
   virtual void onResponseError(folly::exception_wrapper) noexcept = 0;
 
   // If true, the transport can block current thread/fiber until the request is
   // complete.
-  virtual bool isSync() const {
-    return false;
-  }
+  virtual bool isSync() const { return false; }
 
   // If true, the transport can safely run this callback on its internal thread.
   // Should only be used for Thrift internal callbacks.
-  virtual bool isInlineSafe() const {
-    return false;
-  }
+  virtual bool isInlineSafe() const { return false; }
 };
 
 class RequestCallback : public RequestClientCallback {
@@ -277,30 +313,26 @@ class RequestCallback : public RequestClientCallback {
   struct Context {
     bool oneWay{false};
     uint16_t protocolId;
-    std::shared_ptr<apache::thrift::ContextStack> ctx;
+    std::unique_ptr<apache::thrift::ContextStack> ctx;
   };
 
  private:
   friend RequestClientCallback::Ptr toRequestClientCallbackPtr(
-      std::unique_ptr<RequestCallback>,
-      RequestCallback::Context);
+      std::unique_ptr<RequestCallback>, RequestCallback::Context);
 
   void setContext(Context context) {
     context_ = folly::RequestContext::saveContext();
     thriftContext_ = std::move(context);
   }
 
-  void setUnmanaged() {
-    unmanaged_ = true;
-  }
+  void setUnmanaged() { unmanaged_ = true; }
 
   bool unmanaged_{false};
   folly::Optional<Context> thriftContext_;
 };
 
 inline RequestClientCallback::Ptr toRequestClientCallbackPtr(
-    std::unique_ptr<RequestCallback> cb,
-    RequestCallback::Context context) {
+    std::unique_ptr<RequestCallback> cb, RequestCallback::Context context) {
   if (!cb) {
     return RequestClientCallback::Ptr();
   }
@@ -349,12 +381,8 @@ class FunctionSendRecvRequestCallback final : public SendRecvRequestCallback {
   using Recv = folly::Function<void(ClientReceiveState&&)>;
   FunctionSendRecvRequestCallback(Send sendf, Recv recvf)
       : sendf_(std::move(sendf)), recvf_(std::move(recvf)) {}
-  void send(folly::exception_wrapper&& ew) override {
-    sendf_(std::move(ew));
-  }
-  void recv(ClientReceiveState&& state) override {
-    recvf_(std::move(state));
-  }
+  void send(folly::exception_wrapper&& ew) override { sendf_(std::move(ew)); }
+  void recv(ClientReceiveState&& state) override { recvf_(std::move(state)); }
 
  private:
   Send sendf_;
@@ -419,116 +447,19 @@ class CloseCallback {
   virtual ~CloseCallback() {}
 };
 
-/**
- * RpcOptions class to set per-RPC options (such as request timeout).
- */
-class RpcOptions {
- public:
-  typedef apache::thrift::concurrency::PRIORITY PRIORITY;
-  RpcOptions() {}
-
-  /**
-   * NOTE: This only sets the receive timeout, and not the send timeout on
-   * transport. Probably you want to use HeaderClientChannel::setTimeout()
-   */
-  RpcOptions& setTimeout(std::chrono::milliseconds timeout) {
-    timeout_ = timeout;
-    return *this;
-  }
-
-  std::chrono::milliseconds getTimeout() const {
-    return timeout_;
-  }
-
-  RpcOptions& setPriority(PRIORITY priority) {
-    priority_ = static_cast<uint8_t>(priority);
-    return *this;
-  }
-
-  PRIORITY getPriority() const {
-    return static_cast<PRIORITY>(priority_);
-  }
-
-  // Do timeouts apply only on the client side?
-  RpcOptions& setClientOnlyTimeouts(bool val) {
-    clientOnlyTimeouts_ = val;
-    return *this;
-  }
-
-  bool getClientOnlyTimeouts() const {
-    return clientOnlyTimeouts_;
-  }
-
-  RpcOptions& setEnableChecksum(bool val) {
-    enableChecksum_ = val;
-    return *this;
-  }
-
-  bool getEnableChecksum() const {
-    return enableChecksum_;
-  }
-
-  RpcOptions& setChunkTimeout(std::chrono::milliseconds chunkTimeout) {
-    chunkTimeout_ = chunkTimeout;
-    return *this;
-  }
-
-  std::chrono::milliseconds getChunkTimeout() const {
-    return chunkTimeout_;
-  }
-
-  RpcOptions& setChunkBufferSize(int32_t chunkBufferSize) {
-    chunkBufferSize_ = chunkBufferSize;
-    return *this;
-  }
-
-  int32_t getChunkBufferSize() const {
-    return chunkBufferSize_;
-  }
-
-  RpcOptions& setQueueTimeout(std::chrono::milliseconds queueTimeout) {
-    queueTimeout_ = queueTimeout;
-    return *this;
-  }
-
-  std::chrono::milliseconds getQueueTimeout() const {
-    return queueTimeout_;
-  }
-
-  void setWriteHeader(const std::string& key, const std::string& value) {
-    writeHeaders_[key] = value;
-  }
-
-  void setReadHeaders(std::map<std::string, std::string>&& readHeaders) {
-    readHeaders_ = std::move(readHeaders);
-  }
-
-  const std::map<std::string, std::string>& getReadHeaders() const {
-    return readHeaders_;
-  }
-
-  const std::map<std::string, std::string>& getWriteHeaders() const {
-    return writeHeaders_;
-  }
-
-  std::map<std::string, std::string> releaseWriteHeaders() {
-    std::map<std::string, std::string> headers;
-    writeHeaders_.swap(headers);
-    return headers;
-  }
-
- private:
-  std::chrono::milliseconds timeout_{0};
-  std::chrono::milliseconds chunkTimeout_{0};
-  std::chrono::milliseconds queueTimeout_{0};
-  uint8_t priority_{apache::thrift::concurrency::N_PRIORITIES};
-  bool clientOnlyTimeouts_{false};
-  bool enableChecksum_{false};
-  int32_t chunkBufferSize_{0};
-
-  // For sending and receiving headers.
-  std::map<std::string, std::string> writeHeaders_;
-  std::map<std::string, std::string> readHeaders_;
+struct RpcResponseContext {
+  transport::THeader::StringToStringMap headers;
+  RpcSizeStats rpcSizeStats;
+  std::optional<int64_t> serverLoad;
 };
+
+template <class T>
+struct RpcResponseComplete {
+  using response_type = T;
+
+  folly::Try<T> response;
+  RpcResponseContext responseContext;
+};
+
 } // namespace thrift
 } // namespace apache
