@@ -1,11 +1,11 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,11 +16,9 @@
 
 #pragma once
 
+#include <variant>
+
 #include <folly/Portability.h>
-
-#ifdef FOLLY_HAS_COROUTINES
-#include <boost/variant.hpp>
-
 #include <folly/experimental/coro/Baton.h>
 #include <folly/experimental/coro/Task.h>
 
@@ -30,32 +28,28 @@ namespace apache {
 namespace thrift {
 namespace detail {
 
-using ClientMessage = boost::variant<folly::Try<StreamPayload>, int64_t>;
-struct StreamCancel {};
-struct SinkComplete {};
-using ServerMessage =
-    boost::variant<folly::Try<StreamPayload>, StreamCancel, SinkComplete>;
+using ClientMessage = std::variant<folly::Try<StreamPayload>, uint64_t>;
+using ServerMessage = folly::Try<StreamPayload>;
 
-class CoroConsumer {
+class ClientSinkConsumer {
  public:
-  void consume() {
-    baton_.post();
-  }
+  virtual ~ClientSinkConsumer() = default;
+  virtual void consume() = 0;
+  virtual void canceled() = 0;
+};
+#if FOLLY_HAS_COROUTINES
+class CoroConsumer final : public ClientSinkConsumer {
+ public:
+  void consume() { baton_.post(); }
 
-  [[noreturn]] void canceled() {
-    folly::assume_unreachable();
-  }
+  void canceled() { baton_.post(); }
 
-  folly::coro::Task<void> wait() {
-    co_await baton_;
-  }
+  folly::coro::Task<void> wait() { co_await baton_; }
 
  private:
   folly::coro::Baton baton_;
 };
-
+#endif
 } // namespace detail
 } // namespace thrift
 } // namespace apache
-
-#endif
