@@ -1,11 +1,11 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -20,6 +20,7 @@
 #include <unordered_set>
 
 #include <folly/Portability.h>
+#include <folly/experimental/coro/Task.h>
 #include <folly/futures/Future.h>
 #include <thrift/lib/cpp/TApplicationException.h>
 #include <thrift/lib/cpp/protocol/TProtocolTypes.h>
@@ -28,13 +29,28 @@
 #include <thrift/lib/cpp2/async/HeaderChannel.h>
 #include <thrift/lib/cpp2/async/RequestChannel.h>
 
-#if FOLLY_HAS_COROUTINES
-#include <folly/experimental/coro/Task.h>
-#endif
-
 namespace folly {
 
 class IOBuf;
 class IOBufQueue;
 
 } // namespace folly
+
+namespace apache {
+namespace thrift {
+namespace detail {
+namespace si {
+
+enum class InvocationType {
+  AsyncTm,
+  Future,
+  SemiFuture,
+  Sync,
+  CoroParam,
+  Coro,
+};
+
+}
+} // namespace detail
+} // namespace thrift
+} // namespace apache
