@@ -1,11 +1,11 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,9 +13,10 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #pragma once
 
-#include "thrift/lib/cpp/protocol/TType.h"
+#include <thrift/lib/cpp/protocol/TType.h>
 
 namespace apache {
 namespace thrift {
@@ -28,9 +29,7 @@ struct ProtocolReaderWireTypeInfo {
     return ttype;
   }
 
-  static WireType defaultValue() {
-    return protocol::TType::T_STOP;
-  }
+  static WireType defaultValue() { return protocol::TType::T_STOP; }
 };
 
 } // namespace detail
