@@ -1,11 +1,11 @@
 /*
- * Copyright 2017-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,6 +13,8 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
+#include <folly/CPortability.h>
 #include <folly/portability/GTest.h>
 
 #include <thrift/lib/cpp2/protocol/BinaryProtocol.h>
@@ -38,4 +40,24 @@ TEST_F(BinaryProtocolTest, readInvalidBool) {
   EXPECT_THROW(inprot.readBool(value), TProtocolException);
 }
 
+FOLLY_DISABLE_UNDEFINED_BEHAVIOR_SANITIZER("undefined")
+bool makeInvalidBool() {
+  return *reinterpret_cast<const volatile bool*>("\x42");
+}
+
+TEST_F(BinaryProtocolTest, writeInvalidBool) {
+  auto w = BinaryProtocolWriter();
+  auto q = folly::IOBufQueue();
+  w.setOutput(&q);
+  // writeBool should either fail CHECK or write a valid bool.
+  EXPECT_DEATH(
+      {
+        w.writeBool(makeInvalidBool());
+        auto s = std::string();
+        q.appendToString(s);
+        CHECK(s != std::string(1, '\0')); // Die on success.
+      },
+      "Check failed");
+}
+
 } // namespace
