@@ -1,11 +1,11 @@
 /*
- * Copyright 2014-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,49 +13,92 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #include <folly/portability/GTest.h>
 
 #include <folly/SocketAddress.h>
 #include <folly/io/Cursor.h>
 #include <folly/io/IOBuf.h>
 #include <folly/io/IOBufQueue.h>
+#include <folly/io/async/AsyncSSLSocket.h>
+#include <folly/io/async/AsyncSocket.h>
 #include <folly/io/async/AsyncTimeout.h>
+#include <folly/io/async/AsyncTransport.h>
 #include <folly/io/async/EventBase.h>
 #include <folly/io/async/test/SocketPair.h>
 #include <folly/io/async/test/TestSSLServer.h>
 #include <folly/lang/Bits.h>
 #include <thrift/lib/cpp/EventHandlerBase.h>
 #include <thrift/lib/cpp/async/TAsyncSSLSocket.h>
-#include <thrift/lib/cpp/async/TAsyncSocket.h>
 #include <thrift/lib/cpp2/async/Cpp2Channel.h>
 #include <thrift/lib/cpp2/async/HeaderClientChannel.h>
 #include <thrift/lib/cpp2/async/HeaderServerChannel.h>
 #include <thrift/lib/cpp2/async/MessageChannel.h>
+#include <thrift/lib/cpp2/async/RequestCallback.h>
 #include <thrift/lib/cpp2/async/RequestChannel.h>
 #include <thrift/lib/cpp2/async/ResponseChannel.h>
+#include <thrift/lib/cpp2/async/RpcTypes.h>
 
 using namespace apache::thrift;
 using namespace apache::thrift::async;
 using namespace apache::thrift::transport;
 using apache::thrift::ContextStack;
-using std::unique_ptr;
 using folly::IOBuf;
 using folly::IOBufQueue;
-using std::shared_ptr;
 using std::make_unique;
+using std::shared_ptr;
+using std::unique_ptr;
 
-unique_ptr<IOBuf> makeTestBuf(size_t len) {
+unique_ptr<IOBuf> makeTestBufImpl(size_t len) {
   unique_ptr<IOBuf> buf = IOBuf::create(len);
   buf->IOBuf::append(len);
   memset(buf->writableData(), char(0x80), len);
-  return buf;
+  return LegacySerializedRequest(
+             T_COMPACT_PROTOCOL, "test", SerializedRequest(std::move(buf)))
+      .buffer;
+}
+
+unique_ptr<IOBuf> makeTestBuf(size_t len) {
+  for (auto requestLen = len; requestLen > 0; --requestLen) {
+    auto buf = makeTestBufImpl(requestLen);
+    if (buf->computeChainDataLength() == len) {
+      return buf;
+    }
+  }
+  LOG(FATAL) << "Can't generate valid legacy request of given length: " << len;
+}
+
+SerializedRequest makeTestSerializedRequest(size_t len) {
+  for (auto requestLen = len; requestLen > 0; --requestLen) {
+    unique_ptr<IOBuf> buf = IOBuf::create(requestLen);
+    buf->IOBuf::append(requestLen);
+    memset(buf->writableData(), char(0x80), requestLen);
+    if (LegacySerializedRequest(
+            T_COMPACT_PROTOCOL, "test", SerializedRequest(buf->clone()))
+            .buffer->computeChainDataLength() == len) {
+      return SerializedRequest(std::move(buf));
+    }
+  }
+  LOG(FATAL) << "Can't generate valid serialized request of given length: "
+             << len;
+}
+
+size_t lengthWithEnvelope(const ClientReceiveState& state) {
+  return LegacySerializedResponse(
+             state.protocolId(),
+             0,
+             state.messageType(),
+             "test",
+             SerializedResponse(state.serializedResponse().buffer->clone()))
+      .buffer->computeChainDataLength();
 }
 
 class EventBaseAborter : public folly::AsyncTimeout {
  public:
   EventBaseAborter(folly::EventBase* eventBase, uint32_t timeoutMS)
-    : folly::AsyncTimeout(eventBase, folly::AsyncTimeout::InternalEnum::INTERNAL)
-    , eventBase_(eventBase) {
+      : folly::AsyncTimeout(
+            eventBase, folly::AsyncTimeout::InternalEnum::INTERNAL),
+        eventBase_(eventBase) {
     scheduleTimeout(timeoutMS);
   }
 
@@ -70,9 +113,9 @@ class EventBaseAborter : public folly::AsyncTimeout {
 
 // Creates/unwraps a framed message (LEN(MSG) | MSG)
 class TestFramingHandler : public FramingHandler {
-public:
-  std::tuple<unique_ptr<IOBuf>, size_t, unique_ptr<THeader>>
-  removeFrame(IOBufQueue* q) override {
+ public:
+  std::tuple<unique_ptr<IOBuf>, size_t, unique_ptr<THeader>> removeFrame(
+      IOBufQueue* q) override {
     assert(q);
     queue_.append(*q);
     if (!queue_.front() || queue_.front()->empty()) {
@@ -114,23 +157,32 @@ public:
 
     return framing;
   }
-private:
+
+ private:
   IOBufQueue queue_;
 };
 
 template <typename Channel>
 unique_ptr<Channel, folly::DelayedDestruction::Destructor> createChannel(
-    const shared_ptr<TAsyncTransport>& transport) {
-  return Channel::newChannel(transport);
+    folly::AsyncTransport::UniquePtr transport) {
+  return Channel::newChannel(std::move(transport));
 }
 
 template <>
 unique_ptr<Cpp2Channel, folly::DelayedDestruction::Destructor> createChannel(
-    const shared_ptr<TAsyncTransport>& transport) {
-  return Cpp2Channel::newChannel(transport, make_unique<TestFramingHandler>());
+    folly::AsyncTransport::UniquePtr transport) {
+  return Cpp2Channel::newChannel(
+      std::move(transport), make_unique<TestFramingHandler>());
 }
 
-template<typename Channel1, typename Channel2>
+template <>
+HeaderClientChannel::LegacyPtr createChannel(
+    folly::AsyncTransport::UniquePtr transport) {
+  return HeaderClientChannel::newChannel(
+      HeaderClientChannel::WithoutRocketUpgrade{}, std::move(transport));
+}
+
+template <typename Channel1, typename Channel2>
 class SocketPairTest {
  public:
   struct Config {
@@ -140,23 +192,28 @@ class SocketPairTest {
   SocketPairTest(Config config = Config()) : eventBase_() {
     folly::SocketPair socketPair;
 
+    folly::AsyncSocket::UniquePtr socket0, socket1;
     if (config.ssl) {
       auto clientCtx = std::make_shared<folly::SSLContext>();
       auto serverCtx = std::make_shared<folly::SSLContext>();
       getctx(clientCtx, serverCtx);
-      socket0_ = TAsyncSSLSocket::newSocket(
-          clientCtx, &eventBase_, socketPair.extractFD0(), false);
-      socket1_ = TAsyncSSLSocket::newSocket(
-          serverCtx, &eventBase_, socketPair.extractFD1(), true);
-      dynamic_cast<TAsyncSSLSocket*>(socket0_.get())->sslConn(nullptr);
-      dynamic_cast<TAsyncSSLSocket*>(socket1_.get())->sslAccept(nullptr);
+      socket0 = TAsyncSSLSocket::newSocket(
+          clientCtx, &eventBase_, socketPair.extractNetworkSocket0(), false);
+      socket1 = TAsyncSSLSocket::newSocket(
+          serverCtx, &eventBase_, socketPair.extractNetworkSocket1(), true);
+      dynamic_cast<folly::AsyncSSLSocket*>(socket0.get())->sslConn(nullptr);
+      dynamic_cast<folly::AsyncSSLSocket*>(socket1.get())->sslAccept(nullptr);
     } else {
-      socket0_ = TAsyncSocket::newSocket(&eventBase_, socketPair.extractFD0());
-      socket1_ = TAsyncSocket::newSocket(&eventBase_, socketPair.extractFD1());
+      socket0 = folly::AsyncSocket::newSocket(
+          &eventBase_, socketPair.extractNetworkSocket0());
+      socket1 = folly::AsyncSocket::newSocket(
+          &eventBase_, socketPair.extractNetworkSocket1());
     }
+    socket0_ = socket0.get();
+    socket1_ = socket1.get();
 
-    channel0_ = createChannel<Channel1>(socket0_);
-    channel1_ = createChannel<Channel2>(socket1_);
+    channel0_ = createChannel<Channel1>(std::move(socket0));
+    channel1_ = createChannel<Channel2>(std::move(socket1));
   }
   virtual ~SocketPairTest() {}
 
@@ -165,9 +222,7 @@ class SocketPairTest {
     eventBase_.loop();
   }
 
-  void run() {
-    runWithTimeout();
-  }
+  void run() { runWithTimeout(); }
 
   void getctx(
       std::shared_ptr<folly::SSLContext> clientCtx,
@@ -179,21 +234,13 @@ class SocketPairTest {
     serverCtx->loadPrivateKey(folly::kTestKey);
   }
 
-  int getFd0() {
-    return socket0_->getNetworkSocket().toFd();
-  }
+  int getFd0() { return socket0_->getNetworkSocket().toFd(); }
 
-  int getFd1() {
-    return socket1_->getNetworkSocket().toFd();
-  }
+  int getFd1() { return socket1_->getNetworkSocket().toFd(); }
 
-  shared_ptr<TAsyncSocket> getSocket0() {
-    return socket0_;
-  }
+  folly::AsyncSocket* getSocket0() { return socket0_; }
 
-  shared_ptr<TAsyncSocket> getSocket1() {
-    return socket1_;
-  }
+  folly::AsyncSocket* getSocket1() { return socket1_; }
 
   void runWithTimeout(uint32_t timeoutMS = 6000) {
     preLoop();
@@ -206,36 +253,30 @@ class SocketPairTest {
 
  protected:
   folly::EventBase eventBase_;
-  shared_ptr<TAsyncSocket> socket0_;
-  shared_ptr<TAsyncSocket> socket1_;
+  folly::AsyncSocket* socket0_;
+  folly::AsyncSocket* socket1_;
   unique_ptr<Channel1, folly::DelayedDestruction::Destructor> channel0_;
   unique_ptr<Channel2, folly::DelayedDestruction::Destructor> channel1_;
 };
 
-
-class MessageCallback
-    : public MessageChannel::SendCallback
-    , public MessageChannel::RecvCallback {
+class MessageCallback : public MessageChannel::SendCallback,
+                        public MessageChannel::RecvCallback {
  public:
   MessageCallback()
-      : sent_(0)
-      , recv_(0)
-      , sendError_(0)
-      , recvError_(0)
-      , recvEOF_(0)
-      , recvBytes_(0) {}
+      : sent_(0),
+        recv_(0),
+        sendError_(0),
+        recvError_(0),
+        recvEOF_(0),
+        recvBytes_(0) {}
 
   void sendQueued() override {}
 
   void messageSent() override { sent_++; }
-  void messageSendError(folly::exception_wrapper&&) override {
-    sendError_++;
-  }
+  void messageSendError(folly::exception_wrapper&&) override { sendError_++; }
 
   void messageReceived(
-      unique_ptr<IOBuf>&& buf,
-      unique_ptr<THeader>&&,
-      unique_ptr<sample>) override {
+      unique_ptr<IOBuf>&& buf, unique_ptr<THeader>&&) override {
     recv_++;
     recvBytes_ += buf->computeChainDataLength();
   }
@@ -264,7 +305,7 @@ class TestRequestCallback : public RequestClientCallback, public CloseCallback {
 
   void onResponse(ClientReceiveState&& state) noexcept override {
     reply_++;
-    replyBytes_ += state.buf()->computeChainDataLength();
+    replyBytes_ += lengthWithEnvelope(state);
     delete this;
   }
 
@@ -274,9 +315,7 @@ class TestRequestCallback : public RequestClientCallback, public CloseCallback {
     delete this;
   }
 
-  void channelClosed() override {
-    closed_ = true;
-  }
+  void channelClosed() override { closed_ = true; }
 
   static void reset() {
     closed_ = false;
@@ -298,22 +337,19 @@ uint32_t TestRequestCallback::reply_ = 0;
 uint32_t TestRequestCallback::replyBytes_ = 0;
 uint32_t TestRequestCallback::replyError_ = 0;
 
-class ResponseCallback
-    : public ResponseChannel::Callback {
+class ResponseCallback : public HeaderServerChannel::Callback {
  public:
   ResponseCallback()
-      : serverClosed_(false)
-      , oneway_(0)
-      , request_(0)
-      , requestBytes_(0) {}
+      : serverClosed_(false), oneway_(0), request_(0), requestBytes_(0) {}
 
-  void requestReceived(unique_ptr<ResponseChannelRequest>&& req) override {
+  void requestReceived(
+      unique_ptr<HeaderServerChannel::HeaderRequest>&& req) override {
     request_++;
     requestBytes_ += req->getBuf()->computeChainDataLength();
     if (req->isOneway()) {
       oneway_++;
     } else {
-      req->sendReply(req->extractBuf());
+      req->sendReply(ResponsePayload::create(req->extractBuf()));
     }
   }
 
@@ -327,8 +363,8 @@ class ResponseCallback
   uint32_t requestBytes_;
 };
 
-class MessageTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>
-                  , public MessageCallback {
+class MessageTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>,
+                    public MessageCallback {
  public:
   explicit MessageTest(size_t len, Config socketConfig = Config())
       : SocketPairTest(socketConfig), len_(len), header_(new THeader) {}
@@ -347,16 +383,12 @@ class MessageTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>
     EXPECT_EQ(recvBytes_, len_);
   }
 
-  void messageReceived(unique_ptr<IOBuf>&& buf,
-                       unique_ptr<THeader>&& header,
-                       unique_ptr<sample> sample) override {
-    MessageCallback::messageReceived(std::move(buf),
-                                     std::move(header),
-                                     std::move(sample));
+  void messageReceived(
+      unique_ptr<IOBuf>&& buf, unique_ptr<THeader>&& header) override {
+    MessageCallback::messageReceived(std::move(buf), std::move(header));
     channel1_->setReceiveCallback(nullptr);
   }
 
-
  private:
   size_t len_;
   unique_ptr<THeader> header_;
@@ -364,31 +396,30 @@ class MessageTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>
 };
 
 TEST(Channel, Cpp2Channel) {
-  MessageTest(1).run();
+  MessageTest(10).run();
   MessageTest(100).run();
-  MessageTest(1024*1024).run();
+  MessageTest(1024 * 1024).run();
 }
 
 TEST(Channel, Cpp2ChannelSSL) {
   MessageTest::Config socketConfig;
   socketConfig.ssl = true;
-  MessageTest(1, socketConfig).run();
+  MessageTest(10, socketConfig).run();
   MessageTest(100, socketConfig).run();
   MessageTest(1024 * 1024, socketConfig).run();
 }
 
-class MessageCloseTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>
-                       , public MessageCallback {
-public:
+class MessageCloseTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>,
+                         public MessageCallback {
+ public:
   MessageCloseTest() : header_(new THeader) {}
 
   void preLoop() override {
-    channel0_->sendMessage(&sendCallback_,
-                           makeTestBuf(1024*1024),
-                           header_.get());
+    channel0_->sendMessage(
+        &sendCallback_, makeTestBuf(1024 * 1024), header_.get());
     // Close the other socket after delay
     this->eventBase_.runInLoop(
-      std::bind(&TAsyncSocket::close, this->socket1_.get()));
+        std::bind(&folly::AsyncSocket::close, this->socket1_));
     channel1_->setReceiveCallback(this);
   }
 
@@ -414,17 +445,15 @@ TEST(Channel, MessageCloseTest) {
   MessageCloseTest().run();
 }
 
-class MessageEOFTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>
-                     , public MessageCallback {
-public:
+class MessageEOFTest : public SocketPairTest<Cpp2Channel, Cpp2Channel>,
+                       public MessageCallback {
+ public:
   MessageEOFTest() : header_(new THeader) {}
 
   void preLoop() override {
     channel0_->setReceiveCallback(this);
     channel1_->getTransport()->shutdownWrite();
-    channel0_->sendMessage(this,
-                           makeTestBuf(1024*1024),
-                           header_.get());
+    channel0_->sendMessage(this, makeTestBuf(1024 * 1024), header_.get());
   }
 
   void postLoop() override {
@@ -471,12 +500,14 @@ class HeaderChannelTest
     RpcOptions options;
     channel0_->sendRequestNoResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new Callback(this, true)));
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new Callback(this, false)));
     channel0_->setCloseCallback(nullptr);
@@ -489,7 +520,7 @@ class HeaderChannelTest
     EXPECT_EQ(closed_, false);
     EXPECT_EQ(serverClosed_, false);
     EXPECT_EQ(request_, 2);
-    EXPECT_EQ(requestBytes_, len_*2);
+    EXPECT_EQ(requestBytes_, len_ * 2);
     EXPECT_EQ(oneway_, 1);
     channel1_->setCallback(nullptr);
   }
@@ -499,15 +530,15 @@ class HeaderChannelTest
 };
 
 TEST(Channel, HeaderChannelTest) {
-  HeaderChannelTest(1).run();
+  HeaderChannelTest(10).run();
   HeaderChannelTest(100).run();
-  HeaderChannelTest(1024*1024).run();
+  HeaderChannelTest(1024 * 1024).run();
 }
 
 TEST(Channel, HeaderChannelTestSSL) {
   HeaderChannelTest::Config socketConfig;
   socketConfig.ssl = true;
-  HeaderChannelTest(1, socketConfig).run();
+  HeaderChannelTest(10, socketConfig).run();
   HeaderChannelTest(100, socketConfig).run();
   HeaderChannelTest(1024 * 1024, socketConfig).run();
 }
@@ -523,18 +554,12 @@ class HeaderChannelClosedTest
    public:
     explicit Callback(HeaderChannelClosedTest* c) : c_(c) {}
 
-    ~Callback() override {
-      c_->callbackDtor_ = true;
-    }
+    ~Callback() override { c_->callbackDtor_ = true; }
 
     void onResponse(ClientReceiveState&&) noexcept override {
       FAIL() << "should not recv reply from closed channel";
     }
 
-    void onRequestSent() noexcept override {
-      FAIL() << "should not have sent message on closed channel";
-    }
-
     void onResponseError(folly::exception_wrapper ew) noexcept override {
       EXPECT_TRUE(ew.with_exception([this](const TTransportException& e) {
         EXPECT_EQ(e.getType(), TTransportException::END_OF_FILE);
@@ -553,7 +578,8 @@ class HeaderChannelClosedTest
     RpcOptions options;
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(42),
+        "test",
+        makeTestSerializedRequest(42),
         std::make_unique<THeader>(),
         RequestClientCallback::Ptr(new Callback(this)));
   }
@@ -579,7 +605,7 @@ class InOrderTest
       public ResponseCallback {
  public:
   explicit InOrderTest(Config socketConfig = Config())
-      : SocketPairTest(socketConfig), len_(1) {}
+      : SocketPairTest(socketConfig), len_(10) {}
 
   class Callback : public TestRequestCallback {
    public:
@@ -588,19 +614,23 @@ class InOrderTest
       if (reply_ == 1) {
         c_->channel1_->setCallback(nullptr);
         // Verify that they came back in the same order
-        EXPECT_EQ(state.buf()->computeChainDataLength(), c_->len_ + 1);
+        EXPECT_EQ(lengthWithEnvelope(state), c_->len_ + 1);
       }
       TestRequestCallback::onResponse(std::move(state));
     }
 
-    void requestReceived(unique_ptr<ResponseChannelRequest>&& req) {
+    void requestReceived(ResponseChannelRequest::UniquePtr rcr) {
+      auto req = dynamic_cast<HeaderServerChannel::HeaderRequest*>(rcr.get());
       c_->request_++;
       c_->requestBytes_ += req->getBuf()->computeChainDataLength();
       if (c_->firstbuf_) {
-        req->sendReply(req->extractBuf());
-        c_->firstbuf_->sendReply(c_->firstbuf_->extractBuf());
+        req->sendReply(ResponsePayload::create(req->extractBuf()));
+        auto firstbuf = dynamic_cast<HeaderServerChannel::HeaderRequest*>(
+            c_->firstbuf_.get());
+        c_->firstbuf_->sendReply(
+            ResponsePayload::create(firstbuf->extractBuf()));
       } else {
-        c_->firstbuf_ = std::move(req);
+        c_->firstbuf_ = std::move(rcr);
       }
     }
 
@@ -610,17 +640,18 @@ class InOrderTest
 
   void preLoop() override {
     TestRequestCallback::reset();
-    channel0_->setFlags(0); // turn off out of order
     channel1_->setCallback(this);
     RpcOptions options;
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new Callback(this)));
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_ + 1),
+        "test",
+        makeTestSerializedRequest(len_ + 1),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new Callback(this)));
   }
@@ -628,16 +659,16 @@ class InOrderTest
   void postLoop() override {
     EXPECT_EQ(reply_, 2);
     EXPECT_EQ(replyError_, 0);
-    EXPECT_EQ(replyBytes_, 2*len_ + 1);
+    EXPECT_EQ(replyBytes_, 2 * len_ + 1);
     EXPECT_EQ(closed_, false);
     EXPECT_EQ(serverClosed_, false);
     EXPECT_EQ(request_, 2);
-    EXPECT_EQ(requestBytes_, 2*len_ + 1);
+    EXPECT_EQ(requestBytes_, 2 * len_ + 1);
     EXPECT_EQ(oneway_, 0);
   }
 
  private:
-  std::unique_ptr<ResponseChannelRequest> firstbuf_;
+  ResponseChannelRequest::UniquePtr firstbuf_;
   size_t len_;
 };
 
@@ -673,7 +704,8 @@ class BadSeqIdTest
     BadSeqIdTest* c_;
   };
 
-  void requestReceived(unique_ptr<ResponseChannelRequest>&& req) override {
+  void requestReceived(
+      unique_ptr<HeaderServerChannel::HeaderRequest>&& req) override {
     request_++;
     requestBytes_ += req->getBuf()->computeChainDataLength();
     if (req->isOneway()) {
@@ -683,11 +715,8 @@ class BadSeqIdTest
     unique_ptr<THeader> header(new THeader);
     header->setSequenceNumber(-1);
     HeaderServerChannel::HeaderRequest r(
-        channel1_.get(),
-        req->extractBuf(),
-        std::move(header),
-        std::unique_ptr<MessageChannel::RecvCallback::sample>(nullptr));
-    r.sendReply(r.extractBuf());
+        channel1_.get(), req->extractBuf(), std::move(header), {});
+    r.sendReply(ResponsePayload::create(r.extractBuf()));
   }
 
   void preLoop() override {
@@ -697,12 +726,14 @@ class BadSeqIdTest
     RpcOptions options;
     channel0_->sendRequestNoResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new Callback(this, true)));
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new Callback(this, false)));
   }
@@ -714,7 +745,7 @@ class BadSeqIdTest
     EXPECT_EQ(closed_, false);
     EXPECT_EQ(serverClosed_, false);
     EXPECT_EQ(request_, 2);
-    EXPECT_EQ(requestBytes_, len_*2);
+    EXPECT_EQ(requestBytes_, len_ * 2);
     EXPECT_EQ(oneway_, 1);
   }
 
@@ -723,22 +754,22 @@ class BadSeqIdTest
 };
 
 TEST(Channel, BadSeqIdTest) {
-  BadSeqIdTest(1).run();
+  BadSeqIdTest(10).run();
 }
 
 TEST(Channel, BadSeqIdTestSSL) {
   BadSeqIdTest::Config socketConfig;
   socketConfig.ssl = true;
-  BadSeqIdTest(1, socketConfig).run();
+  BadSeqIdTest(10, socketConfig).run();
 }
 
 class TimeoutTest
-    : public SocketPairTest<HeaderClientChannel, HeaderServerChannel>
-    , public TestRequestCallback
-    , public ResponseCallback {
+    : public SocketPairTest<HeaderClientChannel, HeaderServerChannel>,
+      public TestRequestCallback,
+      public ResponseCallback {
  public:
   explicit TimeoutTest(uint32_t timeout, Config socketConfig = Config())
-      : SocketPairTest(socketConfig), timeout_(timeout), len_(1) {}
+      : SocketPairTest(socketConfig), timeout_(timeout), len_(10) {}
 
   void preLoop() override {
     TestRequestCallback::reset();
@@ -748,12 +779,14 @@ class TimeoutTest
     RpcOptions options;
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new TestRequestCallback()));
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new TestRequestCallback()));
   }
@@ -771,17 +804,18 @@ class TimeoutTest
     channel1_->setCallback(nullptr);
   }
 
-  void requestReceived(unique_ptr<ResponseChannelRequest>&& req) override {
+  void requestReceived(
+      unique_ptr<HeaderServerChannel::HeaderRequest>&& req) override {
     request_++;
     requestBytes_ += req->getBuf()->computeChainDataLength();
     // Don't respond, let it time out
     // TestRequestCallback::replyReceived(std::move(buf));
     channel1_->getEventBase()->tryRunAfterDelay(
-      [&](){
-        channel1_->setCallback(nullptr);
-        channel0_->setCloseCallback(nullptr);
-      },
-      timeout_ * 2); // enough time for server socket to close also
+        [&]() {
+          channel1_->setCallback(nullptr);
+          channel0_->setCloseCallback(nullptr);
+        },
+        timeout_ * 2); // enough time for server socket to close also
   }
 
  private:
@@ -810,7 +844,7 @@ class OptionsTimeoutTest
       public ResponseCallback {
  public:
   explicit OptionsTimeoutTest(Config socketConfig = Config())
-      : SocketPairTest(socketConfig), len_(1) {}
+      : SocketPairTest(socketConfig), len_(10) {}
 
   void preLoop() override {
     TestRequestCallback::reset();
@@ -820,7 +854,8 @@ class OptionsTimeoutTest
     options.setTimeout(std::chrono::milliseconds(25));
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(len_),
+        "test",
+        makeTestSerializedRequest(len_),
         std::unique_ptr<THeader>(new THeader),
         RequestClientCallback::Ptr(new TestRequestCallback()));
     // Verify the timeout worked within 10ms
@@ -832,7 +867,8 @@ class OptionsTimeoutTest
           RpcOptions options;
           channel0_->sendRequestResponse(
               options,
-              makeTestBuf(len_),
+              "test",
+              makeTestSerializedRequest(len_),
               std::unique_ptr<THeader>(new THeader),
               RequestClientCallback::Ptr(new TestRequestCallback()));
         },
@@ -842,7 +878,7 @@ class OptionsTimeoutTest
   void postLoop() override {
     EXPECT_EQ(reply_, 1);
     EXPECT_EQ(replyError_, 1);
-    EXPECT_EQ(replyBytes_, 1);
+    EXPECT_EQ(replyBytes_, len_);
     EXPECT_EQ(closed_, false); // client timeouts do not close connection
     EXPECT_EQ(serverClosed_, false);
     EXPECT_EQ(request_, 2);
@@ -850,7 +886,8 @@ class OptionsTimeoutTest
     EXPECT_EQ(oneway_, 0);
   }
 
-  void requestReceived(unique_ptr<ResponseChannelRequest>&& req) override {
+  void requestReceived(
+      unique_ptr<HeaderServerChannel::HeaderRequest>&& req) override {
     if (request_ == 0) {
       request_++;
       requestBytes_ += req->getBuf()->computeChainDataLength();
@@ -875,27 +912,27 @@ TEST(Channel, OptionsTimeoutTestSSL) {
 }
 
 class ClientCloseTest
-    : public SocketPairTest<HeaderClientChannel, HeaderServerChannel>
-    , public TestRequestCallback
-    , public ResponseCallback {
-public:
- explicit ClientCloseTest(bool halfClose) : halfClose_(halfClose) {}
+    : public SocketPairTest<HeaderClientChannel, HeaderServerChannel>,
+      public TestRequestCallback,
+      public ResponseCallback {
+ public:
+  explicit ClientCloseTest(bool halfClose) : halfClose_(halfClose) {}
 
- void preLoop() override {
-   TestRequestCallback::reset();
-   channel1_->setCallback(this);
-   channel0_->setCloseCallback(this);
-   if (halfClose_) {
-     channel1_->getEventBase()->tryRunAfterDelay(
-         [&]() { channel1_->getTransport()->shutdownWrite(); }, 10);
-   } else {
-     channel1_->getEventBase()->tryRunAfterDelay(
-         [&]() { channel1_->getTransport()->close(); }, 10);
-   }
-   channel1_->getEventBase()->tryRunAfterDelay(
-       [&]() { channel1_->setCallback(nullptr); }, 20);
-   channel0_->getEventBase()->tryRunAfterDelay(
-       [&]() { channel0_->setCloseCallback(nullptr); }, 20);
+  void preLoop() override {
+    TestRequestCallback::reset();
+    channel1_->setCallback(this);
+    channel0_->setCloseCallback(this);
+    if (halfClose_) {
+      channel1_->getEventBase()->tryRunAfterDelay(
+          [&]() { channel1_->getTransport()->shutdownWrite(); }, 10);
+    } else {
+      channel1_->getEventBase()->tryRunAfterDelay(
+          [&]() { channel1_->getTransport()->close(); }, 10);
+    }
+    channel1_->getEventBase()->tryRunAfterDelay(
+        [&]() { channel1_->setCallback(nullptr); }, 20);
+    channel0_->getEventBase()->tryRunAfterDelay(
+        [&]() { channel0_->setCloseCallback(nullptr); }, 20);
   }
 
   void postLoop() override {
@@ -919,13 +956,11 @@ TEST(Channel, ClientCloseTest) {
 }
 
 class ServerCloseTest
-    : public SocketPairTest<HeaderClientChannel, HeaderServerChannel>
-    , public TestRequestCallback
-    , public ResponseCallback {
-public:
-  explicit ServerCloseTest(bool halfClose)
-      : halfClose_(halfClose) {
-  }
+    : public SocketPairTest<HeaderClientChannel, HeaderServerChannel>,
+      public TestRequestCallback,
+      public ResponseCallback {
+ public:
+  explicit ServerCloseTest(bool halfClose) : halfClose_(halfClose) {}
 
   void preLoop() override {
     TestRequestCallback::reset();
@@ -933,25 +968,15 @@ public:
     channel0_->setCloseCallback(this);
     if (halfClose_) {
       channel0_->getEventBase()->tryRunAfterDelay(
-        [&](){channel0_->getTransport()->shutdownWrite();
-
-        },
-        10);
+          [&]() { channel0_->getTransport()->shutdownWrite(); }, 10);
     } else {
       channel0_->getEventBase()->tryRunAfterDelay(
-        [&](){channel0_->getTransport()->close();},
-        10);
+          [&]() { channel0_->getTransport()->close(); }, 10);
     }
     channel1_->getEventBase()->tryRunAfterDelay(
-      [&](){
-        channel1_->setCallback(nullptr);
-      },
-    20);
+        [&]() { channel1_->setCallback(nullptr); }, 20);
     channel0_->getEventBase()->tryRunAfterDelay(
-      [&](){
-        channel0_->setCloseCallback(nullptr);
-      },
-    20);
+        [&]() { channel0_->setCloseCallback(nullptr); }, 20);
   }
 
   void postLoop() override {
@@ -975,7 +1000,7 @@ TEST(Channel, ServerCloseTest) {
 }
 
 class ClientCloseOnErrorTest;
-class InvalidResponseCallback : public ResponseChannel::Callback {
+class InvalidResponseCallback : public HeaderServerChannel::Callback {
  public:
   explicit InvalidResponseCallback(ClientCloseOnErrorTest* self)
       : self_(self), request_(0), requestBytes_(0) {}
@@ -986,7 +1011,8 @@ class InvalidResponseCallback : public ResponseChannel::Callback {
     return *this;
   }
 
-  void requestReceived(unique_ptr<ResponseChannelRequest>&& req) override;
+  void requestReceived(
+      unique_ptr<HeaderServerChannel::HeaderRequest>&& req) override;
   void channelClosed(folly::exception_wrapper&&) override {}
 
  protected:
@@ -1044,12 +1070,14 @@ class ClientCloseOnErrorTest
     RpcOptions options;
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(10),
+        "test",
+        makeTestSerializedRequest(10),
         std::make_unique<THeader>(),
         RequestClientCallback::Ptr(new Callback(this)));
     channel0_->sendRequestResponse(
         options,
-        makeTestBuf(reqSize_),
+        "test",
+        makeTestSerializedRequest(reqSize_),
         std::make_unique<THeader>(),
         RequestClientCallback::Ptr(new Callback(this)));
   }
@@ -1069,7 +1097,7 @@ class ClientCloseOnErrorTest
 };
 
 void InvalidResponseCallback::requestReceived(
-    unique_ptr<ResponseChannelRequest>&& req) {
+    unique_ptr<HeaderServerChannel::HeaderRequest>&& req) {
   request_++;
   requestBytes_ += req->getBuf()->computeChainDataLength();
   if (closeSocketInResponse_) {
@@ -1098,27 +1126,29 @@ TEST(Channel, ClientCloseOnErrorTest) {
       .run();
 }
 
-class DestroyAsyncTransport : public apache::thrift::async::TAsyncTransport {
+class DestroyAsyncTransport : public folly::AsyncTransport {
  public:
-  DestroyAsyncTransport() : cb_(nullptr) { }
-  void setReadCB(
-      folly::AsyncTransportWrapper::ReadCallback* callback) override {
+  DestroyAsyncTransport() : cb_(nullptr) {}
+  void setReadCB(folly::AsyncTransport::ReadCallback* callback) override {
     cb_ = callback;
   }
   ReadCallback* getReadCallback() const override {
     return dynamic_cast<ReadCallback*>(cb_);
   }
-  void write(folly::AsyncTransportWrapper::WriteCallback*,
-             const void*,
-             size_t,
-             WriteFlags) override {}
-  void writev(folly::AsyncTransportWrapper::WriteCallback*,
-              const iovec*,
-              size_t,
-              WriteFlags) override {}
-  void writeChain(folly::AsyncTransportWrapper::WriteCallback*,
-                  std::unique_ptr<folly::IOBuf>&&,
-                  WriteFlags) override {}
+  void write(
+      folly::AsyncTransport::WriteCallback*,
+      const void*,
+      size_t,
+      folly::WriteFlags) override {}
+  void writev(
+      folly::AsyncTransport::WriteCallback*,
+      const iovec*,
+      size_t,
+      folly::WriteFlags) override {}
+  void writeChain(
+      folly::AsyncTransport::WriteCallback*,
+      std::unique_ptr<folly::IOBuf>&&,
+      folly::WriteFlags) override {}
   void close() override {}
   void closeNow() override {}
   void shutdownWrite() override {}
@@ -1142,11 +1172,10 @@ class DestroyAsyncTransport : public apache::thrift::async::TAsyncTransport {
   void setEorTracking(bool /* track */) override {}
   bool isEorTrackingEnabled() const override { return false; }
 
-  void invokeEOF() {
-    cb_->readEOF();
-  }
+  void invokeEOF() { cb_->readEOF(); }
+
  private:
-  folly::AsyncTransportWrapper::ReadCallback* cb_;
+  folly::AsyncTransport::ReadCallback* cb_;
 };
 
 class DestroyRecvCallback : public MessageChannel::RecvCallback {
@@ -1154,14 +1183,12 @@ class DestroyRecvCallback : public MessageChannel::RecvCallback {
   typedef std::unique_ptr<Cpp2Channel, folly::DelayedDestruction::Destructor>
       ChannelPointer;
   explicit DestroyRecvCallback(ChannelPointer&& channel)
-      : channel_(std::move(channel)),
-        invocations_(0) {
+      : channel_(std::move(channel)), invocations_(0) {
     channel_->setReceiveCallback(this);
   }
   void messageReceived(
       std::unique_ptr<folly::IOBuf>&&,
-      std::unique_ptr<apache::thrift::transport::THeader>&&,
-      std::unique_ptr<MessageChannel::RecvCallback::sample>) override {}
+      std::unique_ptr<apache::thrift::transport::THeader>&&) override {}
   void messageChannelEOF() override {
     EXPECT_EQ(invocations_, 0);
     invocations_++;
@@ -1174,11 +1201,10 @@ class DestroyRecvCallback : public MessageChannel::RecvCallback {
   int invocations_;
 };
 
-
 TEST(Channel, DestroyInEOF) {
-  auto t = new DestroyAsyncTransport;
-  std::shared_ptr<TAsyncTransport> transport(t);
-  auto channel = createChannel<Cpp2Channel>(transport);
+  auto t = new DestroyAsyncTransport();
+  auto transport = folly::AsyncTransport::UniquePtr(t);
+  auto channel = createChannel<Cpp2Channel>(std::move(transport));
   DestroyRecvCallback drc(std::move(channel));
   t->invokeEOF();
 }
@@ -1187,24 +1213,3 @@ class NullCloseCallback : public CloseCallback {
  public:
   void channelClosed() override {}
 };
-
-TEST(Channel, SetKeepRegisteredForClose) {
-  int lfd = socket(PF_INET, SOCK_STREAM, 0);
-  int rc = listen(lfd, 10);
-  CHECK(rc == 0);
-  struct sockaddr_in addr;
-  socklen_t addrlen = sizeof(addr);
-  rc = getsockname(lfd, (struct sockaddr*)&addr, &addrlen);
-
-  folly::EventBase base;
-  auto transport = TAsyncSocket::newSocket(
-      &base, "127.0.0.1", folly::Endian::big(addr.sin_port));
-  auto channel = createChannel<HeaderClientChannel>(transport);
-  NullCloseCallback ncc;
-  channel->setCloseCallback(&ncc);
-  channel->setKeepRegisteredForClose(false);
-
-  EXPECT_TRUE(base.loop());
-
-  close(lfd);
-}
