@@ -1,11 +1,11 @@
 /*
- * Copyright 2019 Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -16,6 +16,9 @@
 
 #include <exception>
 
+#include <folly/experimental/coro/BlockingWait.h>
+#include <folly/experimental/coro/Coroutine.h>
+#include <folly/io/async/ScopedEventBaseThread.h>
 #include <folly/portability/GTest.h>
 
 #include <thrift/lib/cpp2/test/gen-cpp2/Coroutine.h>
@@ -26,11 +29,11 @@ using apache::thrift::RequestParams;
 using apache::thrift::ScopedServerInterfaceThread;
 using apache::thrift::concurrency::ThreadManager;
 using folly::EventBase;
-using folly::EventBaseManager;
 
 using apache::thrift::test::CoroutineAsyncClient;
 using apache::thrift::test::CoroutineSvIf;
 using apache::thrift::test::CoroutineSvNull;
+using apache::thrift::test::Ex;
 using apache::thrift::test::SumRequest;
 using apache::thrift::test::SumResponse;
 
@@ -42,27 +45,19 @@ static int voidReturnValue;
 class CoroutineServiceHandlerCoro : virtual public CoroutineSvIf {
  public:
   void computeSumNoCoro(
-      SumResponse& response,
-      std::unique_ptr<SumRequest> request) override {
-    response.sum = request->x + request->y;
+      SumResponse& response, std::unique_ptr<SumRequest> request) override {
+    *response.sum_ref() = *request->x_ref() + *request->y_ref();
   }
 
   folly::coro::Task<std::unique_ptr<SumResponse>> co_computeSum(
       std::unique_ptr<SumRequest> request) override {
     auto response = std::make_unique<SumResponse>();
-    response->sum = request->x + request->y;
+    *response->sum_ref() = *request->x_ref() + *request->y_ref();
     co_return response;
   }
 
-  folly::coro::Task<std::unique_ptr<SumResponse>> co_computeSumEb(
-      std::unique_ptr<SumRequest> request) override {
-    auto response = std::make_unique<SumResponse>();
-    response->sum = request->x + request->y;
-    co_return response;
-  }
-
-  folly::coro::Task<int32_t> co_computeSumPrimitive(int32_t x, int32_t y)
-      override {
+  folly::coro::Task<int32_t> co_computeSumPrimitive(
+      int32_t x, int32_t y) override {
     co_return x + y;
   }
 
@@ -73,13 +68,13 @@ class CoroutineServiceHandlerCoro : virtual public CoroutineSvIf {
 
   folly::coro::Task<std::unique_ptr<SumResponse>> co_computeSumThrows(
       std::unique_ptr<SumRequest> /* request */) override {
-    co_await std::experimental::suspend_never{};
+    co_await folly::coro::suspend_never{};
     throw std::runtime_error("Not implemented");
   }
 
-  folly::coro::Task<int32_t> co_computeSumThrowsPrimitive(int32_t, int32_t)
-      override {
-    co_await std::experimental::suspend_never{};
+  folly::coro::Task<int32_t> co_computeSumThrowsPrimitive(
+      int32_t, int32_t) override {
+    co_await folly::coro::suspend_never{};
     throw std::runtime_error("Not implemented");
   }
 
@@ -90,7 +85,7 @@ class CoroutineServiceHandlerCoro : virtual public CoroutineSvIf {
   folly::Future<std::unique_ptr<SumResponse>> future_implementedWithFutures()
       override {
     auto result = std::make_unique<SumResponse>();
-    result->sum = kNoParameterReturnValue;
+    *result->sum_ref() = kNoParameterReturnValue;
     return folly::makeFuture(std::move(result));
   }
 
@@ -116,6 +111,16 @@ class CoroutineServiceHandlerCoro : virtual public CoroutineSvIf {
     co_return;
   }
 
+  folly::coro::Task<std::unique_ptr<SumResponse>> co_computeSumThrowsUserEx(
+      std::unique_ptr<SumRequest>) override {
+    throw Ex();
+  }
+
+  folly::coro::Task<int32_t> co_computeSumThrowsUserExPrimitive(
+      int32_t, int32_t) override {
+    throw Ex();
+  }
+
   folly::Promise<int32_t> onewayRequestPromise;
 };
 
@@ -124,32 +129,24 @@ class CoroutineServiceHandlerCoro : virtual public CoroutineSvIf {
 class CoroutineServiceHandlerFuture : virtual public CoroutineSvIf {
  public:
   void computeSumNoCoro(
-      SumResponse& response,
-      std::unique_ptr<SumRequest> request) override {
-    response.sum = request->x + request->y;
+      SumResponse& response, std::unique_ptr<SumRequest> request) override {
+    *response.sum_ref() = *request->x_ref() + *request->y_ref();
   }
 
   folly::Future<std::unique_ptr<SumResponse>> future_computeSum(
       std::unique_ptr<SumRequest> request) override {
     auto response = std::make_unique<SumResponse>();
-    response->sum = request->x + request->y;
+    *response->sum_ref() = *request->x_ref() + *request->y_ref();
     return folly::makeFuture(std::move(response));
   }
 
-  folly::Future<std::unique_ptr<SumResponse>> future_computeSumEb(
-      std::unique_ptr<SumRequest> request) override {
-    auto response = std::make_unique<SumResponse>();
-    response->sum = request->x + request->y;
-    return folly::makeFuture(std::move(response));
-  }
-
-  folly::Future<int32_t> future_computeSumPrimitive(int32_t x, int32_t y)
-      override {
+  folly::Future<int32_t> future_computeSumPrimitive(
+      int32_t x, int32_t y) override {
     return folly::makeFuture(x + y);
   }
 
-  folly::Future<folly::Unit> future_computeSumVoid(int32_t x, int32_t y)
-      override {
+  folly::Future<folly::Unit> future_computeSumVoid(
+      int32_t x, int32_t y) override {
     voidReturnValue = x + y;
     return folly::makeFuture(folly::Unit{});
   }
@@ -161,8 +158,8 @@ class CoroutineServiceHandlerFuture : virtual public CoroutineSvIf {
             folly::in_place, std::runtime_error("Not implemented")));
   }
 
-  folly::Future<int32_t> future_computeSumThrowsPrimitive(int32_t, int32_t)
-      override {
+  folly::Future<int32_t> future_computeSumThrowsPrimitive(
+      int32_t, int32_t) override {
     return folly::makeFuture<int32_t>(folly::exception_wrapper(
         folly::in_place, std::runtime_error("Not implemented")));
   }
@@ -174,7 +171,7 @@ class CoroutineServiceHandlerFuture : virtual public CoroutineSvIf {
   folly::Future<std::unique_ptr<SumResponse>> future_implementedWithFutures()
       override {
     auto result = std::make_unique<SumResponse>();
-    result->sum = kNoParameterReturnValue;
+    *result->sum_ref() = kNoParameterReturnValue;
     return folly::makeFuture(std::move(result));
   }
 
@@ -193,6 +190,18 @@ class CoroutineServiceHandlerFuture : virtual public CoroutineSvIf {
     return folly::makeFuture(folly::Unit());
   }
 
+  folly::Future<std::unique_ptr<SumResponse>> future_computeSumThrowsUserEx(
+      std::unique_ptr<SumRequest> /* request */) override {
+    return folly::makeFuture<std::unique_ptr<SumResponse>>(
+        folly::exception_wrapper(folly::in_place, Ex()));
+  }
+
+  folly::Future<int32_t> future_computeSumThrowsUserExPrimitive(
+      int32_t, int32_t) override {
+    return folly::makeFuture<int32_t>(
+        folly::exception_wrapper(folly::in_place, Ex()));
+  }
+
   folly::Promise<int32_t> onewayRequestPromise;
 };
 
@@ -202,8 +211,7 @@ class CoroutineTest : public testing::Test {
   CoroutineTest()
       : handler_(std::make_shared<Handler>()),
         ssit_(handler_),
-        client_(ssit_.newClient<CoroutineAsyncClient>(
-            *EventBaseManager::get()->getEventBase())) {}
+        client_(ssit_.newClient<CoroutineAsyncClient>()) {}
 
  protected:
   template <typename Func>
@@ -224,22 +232,22 @@ TYPED_TEST_CASE_P(CoroutineTest);
 TYPED_TEST_P(CoroutineTest, SumNoCoro) {
   this->expectSumResults([&](int x, int y) {
     SumRequest request;
-    request.x = x;
-    request.y = y;
+    *request.x_ref() = x;
+    *request.y_ref() = y;
     SumResponse response;
     this->client_->sync_computeSumNoCoro(response, request);
-    return response.sum;
+    return *response.sum_ref();
   });
 }
 
 TYPED_TEST_P(CoroutineTest, Sum) {
   this->expectSumResults([&](int x, int y) {
     SumRequest request;
-    request.x = x;
-    request.y = y;
+    *request.x_ref() = x;
+    *request.y_ref() = y;
     SumResponse response;
     this->client_->sync_computeSum(response, request);
-    return response.sum;
+    return *response.sum_ref();
   });
 }
 
@@ -256,24 +264,13 @@ TYPED_TEST_P(CoroutineTest, SumVoid) {
   });
 }
 
-TYPED_TEST_P(CoroutineTest, SumEb) {
-  this->expectSumResults([&](int x, int y) {
-    SumRequest request;
-    request.x = x;
-    request.y = y;
-    SumResponse response;
-    this->client_->sync_computeSumEb(response, request);
-    return response.sum;
-  });
-}
-
 TYPED_TEST_P(CoroutineTest, SumUnimplemented) {
   for (int i = 0; i < 10; ++i) {
     bool error = false;
     try {
       SumRequest request;
-      request.x = i;
-      request.y = i;
+      *request.x_ref() = i;
+      *request.y_ref() = i;
       SumResponse response;
       this->client_->sync_computeSumUnimplemented(response, request);
     } catch (...) {
@@ -306,8 +303,8 @@ TYPED_TEST_P(CoroutineTest, SumThrows) {
     bool error = false;
     try {
       SumRequest request;
-      request.x = i;
-      request.y = i;
+      *request.x_ref() = i;
+      *request.y_ref() = i;
       SumResponse response;
       this->client_->sync_computeSumThrows(response, request);
     } catch (...) {
@@ -343,17 +340,17 @@ TYPED_TEST_P(CoroutineTest, NoParameters) {
 
 TYPED_TEST_P(CoroutineTest, ImplemetedWithFutures) {
   SumResponse response;
-  response.sum = 0;
+  *response.sum_ref() = 0;
   this->client_->sync_implementedWithFutures(response);
-  EXPECT_EQ(kNoParameterReturnValue, response.sum);
+  EXPECT_EQ(kNoParameterReturnValue, *response.sum_ref());
 
-  response.sum = 0;
+  *response.sum_ref() = 0;
   this->client_->sync_implementedWithFutures(response);
-  EXPECT_EQ(kNoParameterReturnValue, response.sum);
+  EXPECT_EQ(kNoParameterReturnValue, *response.sum_ref());
 
-  response.sum = 0;
+  *response.sum_ref() = 0;
   this->client_->sync_implementedWithFutures(response);
-  EXPECT_EQ(kNoParameterReturnValue, response.sum);
+  EXPECT_EQ(kNoParameterReturnValue, *response.sum_ref());
 }
 
 TYPED_TEST_P(CoroutineTest, ImplemetedWithFuturesPrimitive) {
@@ -378,13 +375,40 @@ TYPED_TEST_P(CoroutineTest, TakesRequestParams) {
   this->client_->sync_takesRequestParams();
 }
 
+TYPED_TEST_P(CoroutineTest, SumThrowsUserEx) {
+  for (int i = 0; i < 10; ++i) {
+    bool error = false;
+    try {
+      SumRequest request;
+      *request.x_ref() = i;
+      *request.y_ref() = i;
+      SumResponse response;
+      this->client_->sync_computeSumThrowsUserEx(response, request);
+    } catch (const Ex&) {
+      error = true;
+    }
+    EXPECT_TRUE(error);
+  }
+}
+
+TYPED_TEST_P(CoroutineTest, SumThrowsUserExPrimitive) {
+  for (int i = 0; i < 10; ++i) {
+    bool error = false;
+    try {
+      this->client_->sync_computeSumThrowsUserExPrimitive(i, i);
+    } catch (const Ex&) {
+      error = true;
+    }
+    EXPECT_TRUE(error);
+  }
+}
+
 REGISTER_TYPED_TEST_CASE_P(
     CoroutineTest,
     SumNoCoro,
     Sum,
     SumPrimitive,
     SumVoid,
-    SumEb,
     SumUnimplemented,
     SumUnimplementedPrimitive,
     SumThrows,
@@ -393,7 +417,9 @@ REGISTER_TYPED_TEST_CASE_P(
     ImplemetedWithFutures,
     ImplemetedWithFuturesPrimitive,
     Oneway,
-    TakesRequestParams);
+    TakesRequestParams,
+    SumThrowsUserEx,
+    SumThrowsUserExPrimitive);
 
 INSTANTIATE_TYPED_TEST_CASE_P(
     CoroutineTest,
@@ -408,26 +434,25 @@ class CoroutineNullTest : public testing::Test {
  public:
   CoroutineNullTest()
       : ssit_(std::make_shared<CoroutineSvNull>()),
-        client_(ssit_.newClient<CoroutineAsyncClient>(
-            *EventBaseManager::get()->getEventBase())) {}
+        client_(ssit_.newClient<CoroutineAsyncClient>()) {}
   ScopedServerInterfaceThread ssit_;
   std::unique_ptr<CoroutineAsyncClient> client_;
 };
 
 TEST_F(CoroutineNullTest, Basics) {
   SumRequest request;
-  request.x = 123;
-  request.y = 123;
+  *request.x_ref() = 123;
+  *request.y_ref() = 123;
 
   SumResponse response;
 
-  response.sum = 123;
+  *response.sum_ref() = 123;
   client_->sync_computeSumNoCoro(response, request);
-  EXPECT_EQ(0, response.sum);
+  EXPECT_EQ(0, *response.sum_ref());
 
-  response.sum = 123;
+  *response.sum_ref() = 123;
   client_->sync_computeSum(response, request);
-  EXPECT_EQ(0, response.sum);
+  EXPECT_EQ(0, *response.sum_ref());
 
   EXPECT_EQ(0, client_->sync_computeSumPrimitive(123, 456));
 
@@ -441,7 +466,7 @@ class CoroutineClientTest : public testing::Test {
  protected:
   CoroutineClientTest()
       : ssit_(std::make_shared<CoroutineServiceHandlerCoro>()),
-        client_(ssit_.newClient<CoroutineAsyncClient>(eventBase_)) {}
+        client_(ssit_.newClient<CoroutineAsyncClient>()) {}
 
   ScopedServerInterfaceThread ssit_;
   EventBase eventBase_;
@@ -450,14 +475,14 @@ class CoroutineClientTest : public testing::Test {
 
 TEST_F(CoroutineClientTest, SumCoroClient) {
   SumRequest request;
-  request.x = 123;
-  request.y = 123;
+  *request.x_ref() = 123;
+  *request.y_ref() = 123;
 
   client_->co_computeSum(request)
       .semi()
       .via(&eventBase_)
       .then([&](folly::Try<SumResponse> response) {
-        EXPECT_EQ(246, response->sum);
+        EXPECT_EQ(246, *response->sum_ref());
       })
       .getVia(&eventBase_);
 }
@@ -470,20 +495,6 @@ TEST_F(CoroutineClientTest, SumPrimitiveCoroClient) {
       .getVia(&eventBase_);
 }
 
-TEST_F(CoroutineClientTest, SumEbCoroClient) {
-  SumRequest request;
-  request.x = 77;
-  request.y = 941;
-
-  client_->co_computeSumEb(request)
-      .semi()
-      .via(&eventBase_)
-      .then([&](folly::Try<SumResponse> response) {
-        EXPECT_EQ(1018, response->sum);
-      })
-      .getVia(&eventBase_);
-}
-
 TEST_F(CoroutineClientTest, SumVoidCoroClient) {
   client_->co_computeSumVoid(11, 22)
       .semi()
@@ -494,14 +505,14 @@ TEST_F(CoroutineClientTest, SumVoidCoroClient) {
 
 TEST_F(CoroutineClientTest, SumUnimplementedCoroClient) {
   SumRequest request;
-  request.x = 43;
-  request.y = 179;
+  *request.x_ref() = 43;
+  *request.y_ref() = 179;
   client_->co_computeSumUnimplemented(request)
       .semi()
       .via(&eventBase_)
       .then([&](folly::Try<SumResponse> response) {
         EXPECT_THROW(
-            response.throwIfFailed(), apache::thrift::TApplicationException);
+            response.throwUnlessValue(), apache::thrift::TApplicationException);
       })
       .getVia(&eventBase_);
 }
@@ -512,20 +523,20 @@ TEST_F(CoroutineClientTest, SumUnimplementedPrimitiveCoroClient) {
       .via(&eventBase_)
       .then([&](folly::Try<int32_t> response) {
         EXPECT_THROW(
-            response.throwIfFailed(), apache::thrift::TApplicationException);
+            response.throwUnlessValue(), apache::thrift::TApplicationException);
       })
       .getVia(&eventBase_);
 }
 
 TEST_F(CoroutineClientTest, SumThrowsCoroClient) {
   SumRequest request;
-  request.x = 290;
-  request.y = 321;
+  *request.x_ref() = 290;
+  *request.y_ref() = 321;
   client_->co_computeSumThrows(request)
       .semi()
       .via(&eventBase_)
       .then([&](folly::Try<SumResponse> response) {
-        EXPECT_THROW(response.throwIfFailed(), std::exception);
+        EXPECT_THROW(response.throwUnlessValue(), std::exception);
       })
       .getVia(&eventBase_);
 }
@@ -535,7 +546,7 @@ TEST_F(CoroutineClientTest, SumThrowsPrimitiveCoroClient) {
       .semi()
       .via(&eventBase_)
       .then([&](folly::Try<int32_t> response) {
-        EXPECT_THROW(response.throwIfFailed(), std::exception);
+        EXPECT_THROW(response.throwUnlessValue(), std::exception);
       })
       .getVia(&eventBase_);
 }
@@ -555,7 +566,7 @@ TEST_F(CoroutineClientTest, implementedWithFuturesCoroClient) {
       .semi()
       .via(&eventBase_)
       .then([&](folly::Try<SumResponse> response) {
-        EXPECT_EQ(kNoParameterReturnValue, response->sum);
+        EXPECT_EQ(kNoParameterReturnValue, *response->sum_ref());
       })
       .getVia(&eventBase_);
 }
@@ -577,4 +588,136 @@ TEST_F(CoroutineClientTest, takesRequestParamsCoroClient) {
       .then([&](folly::Try<int32_t> result) { EXPECT_EQ(0, *result); })
       .getVia(&eventBase_);
 }
+
+TEST_F(CoroutineClientTest, rpcOptionsCoroClient) {
+  apache::thrift::RpcOptions opts;
+  client_->co_computeSumPrimitive(opts, 12, 408)
+      .semi()
+      .via(&eventBase_)
+      .then([&](folly::Try<int32_t> result) { EXPECT_EQ(420, *result); })
+      .getVia(&eventBase_);
+}
+
+TEST_F(CoroutineClientTest, rpcOptionsCancellableCoroClient) {
+  folly::CancellationSource source;
+  apache::thrift::RpcOptions opts;
+  folly::coro::co_withCancellation(
+      source.getToken(), client_->co_computeSumPrimitive(opts, 12, 408))
+      .semi()
+      .via(&eventBase_)
+      .then([&](folly::Try<int32_t> result) { EXPECT_EQ(420, *result); })
+      .getVia(&eventBase_);
+}
+
+TEST_F(CoroutineClientTest, cancellableCoroClient) {
+  folly::CancellationSource source;
+  folly::coro::co_withCancellation(
+      source.getToken(), client_->co_computeSumPrimitive(12, 408))
+      .semi()
+      .via(&eventBase_)
+      .then([&](folly::Try<int32_t> result) { EXPECT_EQ(420, *result); })
+      .getVia(&eventBase_);
+}
+
+TEST_F(CoroutineClientTest, cancelCoroClient) {
+  folly::CancellationSource source;
+  source.requestCancellation();
+  folly::coro::co_withCancellation(
+      source.getToken(), client_->co_computeSumPrimitive(12, 408))
+      .semi()
+      .via(&eventBase_)
+      .then([&](folly::Try<int32_t> result) {
+        EXPECT_TRUE(result.hasException<folly::OperationCancelled>());
+      })
+      .getVia(&eventBase_);
+}
+
+TEST(CoroutineExceptionTest, completesHandlerCallback) {
+  class CoroutineServiceHandlerThrowing : virtual public CoroutineSvIf {
+   public:
+    folly::coro::Task<std::unique_ptr<SumResponse>> co_computeSumThrows(
+        std::unique_ptr<SumRequest> /* request */) override {
+      throw std::runtime_error("Not implemented");
+    }
+
+    folly::coro::Task<int32_t> co_computeSumThrowsPrimitive(
+        int32_t, int32_t) override {
+      throw std::runtime_error("Not implemented");
+    }
+
+    folly::coro::Task<void> co_onewayRequest(int32_t) override {
+      throw std::runtime_error("Not implemented");
+    }
+  };
+
+  CoroutineServiceHandlerThrowing handler;
+
+  folly::ScopedEventBaseThread ebt;
+  auto tm = ThreadManager::newSimpleThreadManager(1);
+
+  apache::thrift::Cpp2RequestContext cpp2reqCtx(nullptr);
+  auto cb = std::make_unique<
+      apache::thrift::HandlerCallback<std::unique_ptr<SumResponse>>>(
+      nullptr,
+      nullptr,
+      nullptr,
+      nullptr,
+      0,
+      ebt.getEventBase(),
+      tm.get(),
+      &cpp2reqCtx);
+  handler.async_tm_computeSumThrows(std::move(cb), nullptr);
+
+  auto cb2 = std::make_unique<apache::thrift::HandlerCallback<int32_t>>(
+      nullptr,
+      nullptr,
+      nullptr,
+      nullptr,
+      0,
+      ebt.getEventBase(),
+      tm.get(),
+      &cpp2reqCtx);
+  handler.async_tm_computeSumThrowsPrimitive(std::move(cb2), 0, 0);
+
+  auto cb3 = std::make_unique<apache::thrift::HandlerCallbackBase>(
+      nullptr, nullptr, nullptr, ebt.getEventBase(), tm.get(), &cpp2reqCtx);
+  handler.async_tm_onewayRequest(std::move(cb3), 0);
+}
+
+TEST(CoroutineHeaderTest, customHeaderTest) {
+  class CoroHandler : virtual public CoroutineSvIf {
+   public:
+    folly::coro::Task<std::unique_ptr<::apache::thrift::test::SumResponse>>
+    co_computeSum(
+        apache::thrift::RequestParams params,
+        std::unique_ptr<::apache::thrift::test::SumRequest> request) override {
+      if (folly::get_ptr(params.getRequestContext()->getHeaders(), "foo")) {
+        auto header = params.getRequestContext()->getHeader();
+        if (header) {
+          header->setHeader("header_from_server", "1");
+        }
+      }
+      auto response = std::make_unique<SumResponse>();
+      response->sum_ref() = *request->x_ref() + *request->y_ref();
+      co_return response;
+    }
+  };
+
+  std::shared_ptr<CoroHandler> handler = std::make_shared<CoroHandler>();
+  ScopedServerInterfaceThread ssit{handler};
+  auto client = ssit.newClient<CoroutineAsyncClient>();
+
+  apache::thrift::RpcOptions rpcOptions;
+  rpcOptions.setWriteHeader("foo", "bar");
+
+  SumRequest sumRequest;
+  sumRequest.x_ref() = 42;
+  sumRequest.y_ref() = 123;
+  auto result =
+      folly::coro::blockingWait(client->co_computeSum(rpcOptions, sumRequest));
+  auto ptr = folly::get_ptr(rpcOptions.getReadHeaders(), "header_from_server");
+  EXPECT_NE(nullptr, ptr);
+  EXPECT_EQ("1", *ptr);
+  EXPECT_EQ(165, *result.sum_ref());
+}
 #endif
