@@ -1,11 +1,11 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,18 +13,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#include <thrift/lib/cpp2/transport/rocket/PayloadUtils.h>
 
-#include <thrift/lib/thrift/gen-cpp2/RpcMetadata_types.h>
+#include <thrift/lib/cpp2/transport/rocket/PayloadUtils.h>
 
 namespace apache {
 namespace thrift {
 namespace rocket {
+namespace detail {
 
 template <class Metadata>
 Payload makePayload(
-    const Metadata& metadata,
-    std::unique_ptr<folly::IOBuf> data) {
+    const Metadata& metadata, std::unique_ptr<folly::IOBuf> data) {
   CompactProtocolWriter writer;
   // Default is to leave some headroom for rsocket headers
   size_t serSize = metadata.serializedSizeZC(&writer);
@@ -33,8 +32,8 @@ Payload makePayload(
   folly::IOBufQueue queue;
 
   // If possible, serialize metadata into the headeroom of data.
-  if (!data->isChained() && data->headroom() >= serSize + kHeadroomBytes &&
-      !data->isSharedOne()) {
+  if (data && !data->isChained() &&
+      data->headroom() >= serSize + kHeadroomBytes && !data->isSharedOne()) {
     // Store previous state of the buffer pointers and rewind it.
     auto startBuffer = data->buffer();
     auto start = data->data();
@@ -67,28 +66,59 @@ Payload makePayload(
 }
 
 template Payload makePayload<>(
-    const RequestRpcMetadata&,
-    std::unique_ptr<folly::IOBuf> data);
+    const RequestRpcMetadata&, std::unique_ptr<folly::IOBuf> data);
 template Payload makePayload<>(
-    const ResponseRpcMetadata&,
-    std::unique_ptr<folly::IOBuf> data);
+    const ResponseRpcMetadata&, std::unique_ptr<folly::IOBuf> data);
+template Payload makePayload<>(
+    const StreamPayloadMetadata&, std::unique_ptr<folly::IOBuf> data);
+template Payload makePayload<>(
+    const HeadersPayloadMetadata&, std::unique_ptr<folly::IOBuf> data);
 
-/**
- * Helper method to compress the request before sending to the server.
- */
-void compressRequest(
+template <typename Metadata>
+void setCompressionCodec(
+    CompressionConfig compressionConfig,
+    Metadata& metadata,
+    size_t payloadSize) {
+  if (auto codecRef = compressionConfig.codecConfig_ref()) {
+    if (payloadSize >
+        static_cast<size_t>(
+            compressionConfig.compressionSizeLimit_ref().value_or(0))) {
+      switch (codecRef->getType()) {
+        case CodecConfig::zlibConfig:
+          metadata.compression_ref() = CompressionAlgorithm::ZLIB;
+          break;
+        case CodecConfig::zstdConfig:
+          metadata.compression_ref() = CompressionAlgorithm::ZSTD;
+          break;
+        default:
+          break;
+      }
+    }
+  }
+}
+
+template void setCompressionCodec<>(
+    CompressionConfig compressionConfig,
     RequestRpcMetadata& metadata,
-    std::unique_ptr<folly::IOBuf>& data,
-    CompressionAlgorithm compression) {
+    size_t payloadSize);
+template void setCompressionCodec<>(
+    CompressionConfig compressionConfig,
+    ResponseRpcMetadata& metadata,
+    size_t payloadSize);
+template void setCompressionCodec<>(
+    CompressionConfig compressionConfig,
+    StreamPayloadMetadata& metadata,
+    size_t payloadSize);
+
+void compressPayload(
+    std::unique_ptr<folly::IOBuf>& data, CompressionAlgorithm compression) {
   folly::io::CodecType codec;
   switch (compression) {
     case CompressionAlgorithm::ZSTD:
       codec = folly::io::CodecType::ZSTD;
-      metadata.compression_ref() = compression;
       break;
     case CompressionAlgorithm::ZLIB:
       codec = folly::io::CodecType::ZLIB;
-      metadata.compression_ref() = compression;
       break;
     case CompressionAlgorithm::NONE:
       codec = folly::io::CodecType::NO_COMPRESSION;
@@ -97,9 +127,8 @@ void compressRequest(
   data = folly::io::getCodec(codec)->compress(data.get());
 }
 
-void uncompressRequest(
-    CompressionAlgorithm compression,
-    std::unique_ptr<folly::IOBuf>& data) {
+folly::Expected<std::unique_ptr<folly::IOBuf>, std::string> uncompressPayload(
+    CompressionAlgorithm compression, std::unique_ptr<folly::IOBuf> data) {
   folly::io::CodecType codec;
   switch (compression) {
     case CompressionAlgorithm::ZSTD:
@@ -112,7 +141,23 @@ void uncompressRequest(
       codec = folly::io::CodecType::NO_COMPRESSION;
       break;
   }
-  data = folly::io::getCodec(codec)->uncompress(data.get());
+
+  try {
+    return folly::io::getCodec(codec)->uncompress(data.get());
+  } catch (const std::exception& e) {
+    return folly::makeUnexpected(std::string(e.what()));
+  }
+}
+} // namespace detail
+std::unique_ptr<folly::IOBuf> uncompressBuffer(
+    std::unique_ptr<folly::IOBuf>&& buffer, CompressionAlgorithm compression) {
+  auto result = detail::uncompressPayload(compression, std::move(buffer));
+  if (!result) {
+    folly::throw_exception<TApplicationException>(
+        TApplicationException::INVALID_TRANSFORM,
+        fmt::format("decompression failure: {}", std::move(result.error())));
+  }
+  return std::move(result.value());
 }
 } // namespace rocket
 } // namespace thrift
