@@ -1,11 +1,11 @@
 /*
- * Copyright 2018-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -40,7 +40,15 @@ enum class FrameType : uint8_t {
   METADATA_PUSH = 0x0C,
   // RESUME = 0x0D,
   // RESUME_OK = 0x0E,
-  // EXT = 0x3F,
+  EXT = 0x3F,
+};
+
+enum class ExtFrameType : uint32_t {
+  UNKNOWN = 0x00, // Never transmitted over the wire.
+  HEADERS_PUSH = 0xFB0,
+  ALIGNED_PAGE = 0xFB1,
+  INTERACTION_TERMINATE = 0xFB2,
+  CUSTOM_ALLOC = 0xFB3,
 };
 
 } // namespace rocket
