@@ -1,11 +1,11 @@
 /*
- * Copyright 2018-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -36,28 +36,31 @@ using namespace apache::thrift::rocket;
 using namespace apache::thrift::rocket::test;
 
 namespace {
-constexpr StreamId kTestStreamId{178};
+constexpr StreamId kTestStreamId{179};
 constexpr folly::StringPiece kMetadata{"metadata"};
 constexpr folly::StringPiece kData{"data"};
 
 template <class Frame>
-Frame serializeAndDeserialize(Frame&& frame) {
+std::unique_ptr<folly::IOBuf> serialize(Frame frame) {
   Serializer writer;
-  std::forward<Frame>(frame).serialize(writer);
+  std::move(frame).serialize(writer);
   auto serializedFrameData = std::move(writer).move();
   serializedFrameData->coalesce();
+  return serializedFrameData;
+}
+
+template <class Frame>
+Frame serializeAndDeserialize(Frame frame) {
+  auto serializedFrameData = serialize(std::move(frame));
+  EXPECT_TRUE(isMaybeRocketFrame(*serializedFrameData));
   // Skip past frame length
   serializedFrameData->trimStart(Serializer::kBytesForFrameOrMetadataLength);
   return Frame(std::move(serializedFrameData));
 }
 
 template <class Frame>
-Frame serializeAndDeserializeFragmented(Frame&& frame) {
-  Serializer writer;
-  std::forward<Frame>(frame).serialize(writer);
-  auto serializedFrameData = std::move(writer).move();
-  serializedFrameData->coalesce();
-
+Frame serializeAndDeserializeFragmented(Frame frame) {
+  auto serializedFrameData = serialize(std::move(frame));
   folly::Optional<Frame> returnFrame;
   folly::io::Cursor cursor(serializedFrameData.get());
   bool hitSplitLogic = false;
@@ -137,6 +140,31 @@ TEST(FrameSerialization, RequestResponseSanity) {
   validate(serializeAndDeserialize(std::move(frame)));
 }
 
+TEST(FrameSerialization, RequestResponseNoHeadroomWriteForSharedBuf) {
+  // Verify headroom write not attampted on shared IOBufs
+  const std::string str = "12345678901234567890abcdef";
+  auto dt = folly::IOBuf::wrapBuffer(str.data(), str.size());
+  auto md = dt->clone();
+  // share the same underlying buffer, but leave enough headroom for
+  // frame header, if it was attempted to be written to the headroom
+  md->trimStart(20);
+  RequestResponseFrame frame(
+      kTestStreamId,
+      Payload::makeFromMetadataAndData(std::move(md), std::move(dt)));
+
+  auto serialBuf = std::move(frame).serialize();
+  // deserialize
+  serialBuf->coalesce();
+  serialBuf->trimStart(Serializer::kBytesForFrameOrMetadataLength);
+  RequestResponseFrame frame2(std::move(serialBuf));
+
+  auto dataAndMetadata = splitMetadataAndData(frame2.payload());
+  // Explicitly check copies in comparison, in case of erroneous write
+  // to headroom original input data would be overwritten.
+  EXPECT_EQ("abcdef", getRange(*dataAndMetadata.first));
+  EXPECT_EQ("12345678901234567890abcdef", getRange(*dataAndMetadata.second));
+}
+
 TEST(FrameSerialization, RequestFnfSanity) {
   RequestFnfFrame frame(
       kTestStreamId, Payload::makeFromMetadataAndData(kMetadata, kData));
@@ -214,7 +242,7 @@ TEST(FrameSerialization, PayloadSanity) {
   PayloadFrame frame(
       kTestStreamId,
       Payload::makeFromMetadataAndData(kMetadata, kData),
-      Flags::none().follows(true).complete(true).next(true));
+      Flags().follows(true).complete(true).next(true));
 
   auto validate = [](const PayloadFrame& f) {
     EXPECT_EQ(kTestStreamId, f.streamId());
@@ -244,7 +272,7 @@ TEST(FrameSerialization, PayloadEmptyMetadataSanity) {
     PayloadFrame frame(
         kTestStreamId,
         Payload::makeFromData(kData),
-        Flags::none().follows(true).complete(true).next(true));
+        Flags().follows(true).complete(true).next(true));
 
     validate(frame);
     validate(serializeAndDeserialize(std::move(frame)));
@@ -256,7 +284,7 @@ TEST(FrameSerialization, PayloadEmptyMetadataSanity) {
         kTestStreamId,
         Payload::makeFromMetadataAndData(
             folly::ByteRange{folly::StringPiece{""}}, kData),
-        Flags::none().follows(true).complete(true).next(true));
+        Flags().follows(true).complete(true).next(true));
 
     validate(frame);
     validate(serializeAndDeserialize(std::move(frame)));
@@ -309,8 +337,7 @@ TEST(FrameSerialization, MetadataPushSanity) {
     wcursor.writeBE<uint32_t>(0);
     // write frameType and flags
     wcursor.writeBE<uint16_t>(
-        static_cast<uint8_t>(FrameType::METADATA_PUSH)
-            << Flags::frameTypeOffset() |
+        static_cast<uint8_t>(FrameType::METADATA_PUSH) << Flags::kBits |
         static_cast<uint16_t>(1 << 8));
 
     wcursor.insert(folly::IOBuf::copyBuffer(kMeta));
@@ -331,7 +358,7 @@ TEST(FrameSerialization, KeepAliveSanity) {
 
   auto makeKeepAliveFrame = [&] {
     return KeepAliveFrame(
-        Flags::none().respond(true), folly::IOBuf::copyBuffer(kKeepAliveData));
+        Flags().respond(true), folly::IOBuf::copyBuffer(kKeepAliveData));
   };
 
   auto validate = [=](KeepAliveFrame&& f) {
@@ -343,6 +370,27 @@ TEST(FrameSerialization, KeepAliveSanity) {
   validate(serializeAndDeserialize(makeKeepAliveFrame()));
 }
 
+TEST(FrameSerialization, KeepAliveSanityEmptyPayload) {
+  constexpr folly::StringPiece kKeepAliveData = folly::StringPiece();
+
+  auto makeKeepAliveFrame = [&] {
+    return KeepAliveFrame(
+        Flags().respond(true), folly::IOBuf::copyBuffer(kKeepAliveData));
+  };
+
+  auto validate = [=](KeepAliveFrame&& f) {
+    EXPECT_TRUE(f.hasRespondFlag());
+    EXPECT_EQ(kKeepAliveData, getRange(*std::move(f).data()));
+  };
+  auto validateSerialized = [=](KeepAliveFrame&& f) {
+    EXPECT_TRUE(f.hasRespondFlag());
+    EXPECT_EQ(nullptr, std::move(f).data());
+  };
+
+  validate(makeKeepAliveFrame());
+  validateSerialized(serializeAndDeserialize(makeKeepAliveFrame()));
+}
+
 TEST(FrameSerialization, PayloadLargeMetadata) {
   const auto metadata = makeLargeIOBuf();
 
@@ -350,7 +398,7 @@ TEST(FrameSerialization, PayloadLargeMetadata) {
       kTestStreamId,
       Payload::makeFromMetadataAndData(
           metadata.clone(), folly::IOBuf::copyBuffer(kData)),
-      Flags::none().complete(true).next(true));
+      Flags().complete(true).next(true));
 
   auto validate = [=](const PayloadFrame& f) {
     EXPECT_EQ(kTestStreamId, f.streamId());
@@ -371,7 +419,7 @@ TEST(FrameSerialization, PayloadLargeData) {
       kTestStreamId,
       Payload::makeFromMetadataAndData(
           folly::IOBuf::copyBuffer(kMetadata), data.clone()),
-      Flags::none().complete(true).next(true));
+      Flags().complete(true).next(true));
 
   auto validate = [=](const PayloadFrame& f) {
     EXPECT_EQ(kTestStreamId, f.streamId());
@@ -560,11 +608,11 @@ TEST(FrameSerialization, PayloadFrameSerializeAPI) {
     PayloadFrame frame1(
         kTestStreamId,
         Payload::makeFromMetadataAndData(md->clone(), data->clone()),
-        Flags::none().follows(true).complete(true).next(true));
+        Flags().follows(true).complete(true).next(true));
     PayloadFrame frame2(
         kTestStreamId,
         Payload::makeFromMetadataAndData(md->clone(), data->clone()),
-        Flags::none().follows(true).complete(true).next(true));
+        Flags().follows(true).complete(true).next(true));
     auto altApi = std::move(frame1).serialize();
     Serializer writer;
     std::move(frame2).serialize(writer);
@@ -597,6 +645,77 @@ TEST(FrameSerialization, PayloadFrameSerializeAPI) {
   validate(std::make_unique<folly::IOBuf>(), folly::IOBuf::wrapBuffer(kData));
 }
 
+TEST(FrameSerialization, ExtSanity) {
+  ExtFrame frame(
+      kTestStreamId,
+      Payload::makeFromMetadataAndData(kMetadata, kData),
+      Flags().ignore(true),
+      ExtFrameType::HEADERS_PUSH);
+
+  auto validate = [](const ExtFrame& f) {
+    EXPECT_EQ(kTestStreamId, f.streamId());
+    EXPECT_EQ(ExtFrameType::HEADERS_PUSH, f.extFrameType());
+    EXPECT_TRUE(f.hasIgnore());
+    validateMetadataAndData(f.payload());
+  };
+
+  validate(frame);
+  validate(serializeAndDeserialize(std::move(frame)));
+}
+
+TEST(FrameSerialization, ExtUnknownSanity) {
+  ExtFrame frame(
+      kTestStreamId,
+      Payload::makeFromMetadataAndData(kMetadata, kData),
+      Flags().ignore(true),
+      static_cast<ExtFrameType>(42));
+
+  auto validate = [](const ExtFrame& f) {
+    EXPECT_EQ(kTestStreamId, f.streamId());
+    EXPECT_EQ(ExtFrameType::UNKNOWN, f.extFrameType());
+    EXPECT_TRUE(f.hasIgnore());
+    validateMetadataAndData(f.payload());
+  };
+
+  validate(serializeAndDeserialize(std::move(frame)));
+}
+
+TEST(FrameSerialization, ExtEmptyMetadataSanity) {
+  auto validate = [](const ExtFrame& f) {
+    EXPECT_EQ(kTestStreamId, f.streamId());
+    EXPECT_EQ(ExtFrameType::HEADERS_PUSH, f.extFrameType());
+    EXPECT_TRUE(f.hasIgnore());
+    EXPECT_FALSE(f.payload().hasNonemptyMetadata());
+    auto dam = splitMetadataAndData(f.payload());
+    EXPECT_EQ(kData, getRange(*dam.second));
+  };
+
+  // No metadata
+  {
+    ExtFrame frame(
+        kTestStreamId,
+        Payload::makeFromData(kData),
+        Flags().ignore(true),
+        ExtFrameType::HEADERS_PUSH);
+
+    validate(frame);
+    validate(serializeAndDeserialize(std::move(frame)));
+  }
+
+  // Empty metadata
+  {
+    ExtFrame frame(
+        kTestStreamId,
+        Payload::makeFromMetadataAndData(
+            folly::ByteRange{folly::StringPiece{""}}, kData),
+        Flags().ignore(true),
+        ExtFrameType::HEADERS_PUSH);
+
+    validate(frame);
+    validate(serializeAndDeserialize(std::move(frame)));
+  }
+}
+
 } // namespace rocket
 } // namespace thrift
 } // namespace apache
