@@ -1,21 +1,19 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements. See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership. The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License. You may obtain a copy of the License at
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
- * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied. See the License for the
- * specific language governing permissions and limitations
- * under the License.
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+
 package com.facebook.thrift.transport;
 
 import com.facebook.thrift.TApplicationException;
@@ -24,12 +22,13 @@ import com.facebook.thrift.protocol.TBinaryProtocol;
 import com.facebook.thrift.protocol.TCompactProtocol;
 import com.facebook.thrift.protocol.TMessage;
 import com.facebook.thrift.protocol.TMessageType;
+import com.facebook.thrift.utils.StandardCharsets;
+import java.io.IOException;
+import java.net.URL;
 import java.nio.ByteBuffer;
-import java.nio.charset.StandardCharsets;
-import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
+import java.util.jar.Attributes;
+import java.util.jar.Manifest;
 import java.util.zip.DataFormatException;
 import java.util.zip.Deflater;
 import java.util.zip.Inflater;
@@ -51,9 +50,7 @@ public class THeaderTransport extends TFramedTransport {
 
   // Transforms
   public enum Transforms {
-    ZLIB_TRANSFORM(0x01),
-    HMAC_TRANSFORM(0x02), // Deprecated and no longer supported
-    SNAPPY_TRANSFORM(0x03); // Deprecated and no longer supported
+    ZLIB_TRANSFORM(0x01);
 
     private int value;
 
@@ -86,7 +83,6 @@ public class THeaderTransport extends TFramedTransport {
   public enum ClientTypes {
     HEADERS(0),
     FRAMED_DEPRECATED(1),
-    UNFRAMED_DEPRECATED(2),
     HTTP(3),
     UNKNOWN(4);
 
@@ -125,9 +121,14 @@ public class THeaderTransport extends TFramedTransport {
   private static final String IDENTITY_HEADER = "identity";
   private static final String ID_VERSION_HEADER = "id_version";
   private static final String ID_VERSION = "1";
+  private static final String FBCODE_BUILD_RULE_MANIFEST_ATTR = "Fbcode-Build-Rule";
+  private static final String CLIENT_METADATA_HEADER = "client_metadata";
+  private static final String CLIENT_METADATA = getClientMetadata();
 
   private String identity;
 
+  private boolean firstRequest = true;
+
   public THeaderTransport(TTransport transport) {
     super(transport);
     writeTransforms = new ArrayList<Transforms>();
@@ -173,6 +174,20 @@ public class THeaderTransport extends TFramedTransport {
     this.protoId = protoId;
   }
 
+  /**
+   * Returns the current client type we are reading/writing
+   *
+   * @return client type
+   */
+  public ClientTypes getClientType() {
+    return this.clientType;
+  }
+
+  /** Sets client type we are writing */
+  public void setClientType(ClientTypes clientType) {
+    this.clientType = clientType;
+  }
+
   /**
    * Sets the internal buffer size for zlib transform This will work with any value (except 0), but
    * this is provided as an optimization knob.
@@ -242,10 +257,6 @@ public class THeaderTransport extends TFramedTransport {
       }
     }
 
-    if (clientType == ClientTypes.UNFRAMED_DEPRECATED) {
-      return transport_.read(buf, off, len);
-    }
-
     // Read another frame of data
     readFrame(len);
 
@@ -275,15 +286,7 @@ public class THeaderTransport extends TFramedTransport {
     int word1 = decodeWord(i32buf);
 
     if ((word1 & TBinaryProtocol.VERSION_MASK) == TBinaryProtocol.VERSION_1) {
-      clientType = ClientTypes.UNFRAMED_DEPRECATED;
-      if (reqLen <= 4) {
-        readBuffer_.reset(i32buf);
-      } else {
-        byte[] buff = new byte[reqLen];
-        System.arraycopy(i32buf, 0, buff, 0, 4);
-        transport_.readAll(buff, 4, reqLen - 4);
-        readBuffer_.reset(buff);
-      }
+      throw new THeaderException("This transport does not support Unframed");
     } else if (word1 == HTTP_SERVER_MAGIC) {
       throw new THeaderException("This transport does not support HTTP");
     } else {
@@ -390,10 +393,10 @@ public class THeaderTransport extends TFramedTransport {
       throw new TTransportException("Header size is larger than frame");
     }
     protoId = readVarint32Buf(frame);
-    int numHeaders = readVarint32Buf(frame);
+    int numTransforms = readVarint32Buf(frame);
 
     // Clear out any previous transforms
-    readTransforms = new ArrayList<Integer>(numHeaders);
+    readTransforms = new ArrayList<Integer>(numTransforms);
 
     if (protoId == T_JSON_PROTOCOL && clientType != ClientTypes.HTTP) {
       throw new TTransportException("Trying to recv JSON encoding " + "over binary");
@@ -402,14 +405,10 @@ public class THeaderTransport extends TFramedTransport {
     // Read in the headers.  Data for each varies. See
     // doc/HeaderFormat.txt
     int hmacSz = 0;
-    for (int i = 0; i < numHeaders; i++) {
+    for (int i = 0; i < numTransforms; i++) {
       int transId = readVarint32Buf(frame);
       if (transId == Transforms.ZLIB_TRANSFORM.getValue()) {
         readTransforms.add(transId);
-      } else if (transId == Transforms.SNAPPY_TRANSFORM.getValue()) {
-        throw new THeaderException("Snappy transform no longer supported");
-      } else if (transId == Transforms.HMAC_TRANSFORM.getValue()) {
-        throw new THeaderException("Hmac transform no longer supported");
       } else {
         throw new THeaderException("Unknown transform during recv");
       }
@@ -438,6 +437,7 @@ public class THeaderTransport extends TFramedTransport {
         break;
       }
     }
+    readHeaders.remove(CLIENT_METADATA_HEADER);
     readHeaders.putAll(readPersistentHeaders);
 
     // Read in the data section.
@@ -542,7 +542,7 @@ public class THeaderTransport extends TFramedTransport {
   }
 
   /* Writes the output buffer in header format, or format
-   * client responded with (framed, unframed, http)
+   * client responded with (framed, http)
    */
   @Override
   public void flush() throws TTransportException {
@@ -621,6 +621,11 @@ public class THeaderTransport extends TFramedTransport {
       transformData.limit(transformData.position());
       transformData.position(0);
 
+      if (firstRequest) {
+        firstRequest = false;
+        writeHeaders.put(CLIENT_METADATA_HEADER, CLIENT_METADATA);
+      }
+
       if (identity != null && identity.length() > 0) {
         writeHeaders.put(ID_VERSION_HEADER, ID_VERSION);
         writeHeaders.put(IDENTITY_HEADER, identity);
@@ -674,8 +679,6 @@ public class THeaderTransport extends TFramedTransport {
       out.position(0);
       transport_.write(out.array(), out.position(), out.remaining());
       transport_.write(frame.array(), frame.position(), frame.remaining());
-    } else if (clientType == ClientTypes.UNFRAMED_DEPRECATED) {
-      transport_.write(frame.array(), frame.position(), frame.remaining());
     } else if (clientType == ClientTypes.HTTP) {
       throw new TTransportException("HTTP is unimplemented in this language");
     } else {
@@ -718,12 +721,27 @@ public class THeaderTransport extends TFramedTransport {
     }
   }
 
-  @Override
-  public int getBytesRemainingInBuffer() {
-    if (supportedClients[ClientTypes.UNFRAMED_DEPRECATED.getValue()]) {
-      return transport_.getBytesRemainingInBuffer();
-    } else {
-      return super.getBytesRemainingInBuffer();
+  private static String getClientMetadata() {
+    return String.format(
+        "{\"agent\":\"nifty.THeaderTransport.java\",\"clientMetadata\":{\"build_rule\":\"%s\"}}",
+        getClientBuildRule());
+  }
+
+  private static String getClientBuildRule() {
+    try {
+      Enumeration<URL> resources =
+          THeaderTransport.class.getClassLoader().getResources("META-INF/MANIFEST.MF");
+      while (resources.hasMoreElements()) {
+        Manifest manifest = new Manifest(resources.nextElement().openStream());
+        Attributes attr = manifest.getMainAttributes();
+        if (attr.containsKey(FBCODE_BUILD_RULE_MANIFEST_ATTR)) {
+          return attr.getValue("Fbcode-Build-Rule");
+        }
+      }
+    } catch (IOException e) {
+      return "<unknown_build_rule>";
     }
+
+    return "<unknown_build_rule>";
   }
 }
