@@ -1,21 +1,18 @@
+# Copyright (c) Facebook, Inc. and its affiliates.
 #
-# Licensed to the Apache Software Foundation (ASF) under one
-# or more contributor license agreements. See the NOTICE file
-# distributed with this work for additional information
-# regarding copyright ownership. The ASF licenses this file
-# to you under the Apache License, Version 2.0 (the
-# "License"); you may not use this file except in compliance
-# with the License. You may obtain a copy of the License at
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-#   http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing,
-# software distributed under the License is distributed on an
-# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-# KIND, either express or implied. See the License for the
-# specific language governing permissions and limitations
-# under the License.
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+# pyre-unsafe
 
 from __future__ import absolute_import
 from __future__ import division
@@ -25,12 +22,13 @@ from __future__ import unicode_literals
 import sys
 if sys.version_info[0] >= 3:
     from http import server
+    # pyre-fixme[11]: Annotation `server` is not defined as a type.
     BaseHTTPServer = server
     xrange = range
     from io import BytesIO as StringIO
     PY3 = True
 else:
-    import BaseHTTPServer
+    import BaseHTTPServer  # @manual
     from cStringIO import StringIO
     PY3 = False
 
@@ -39,7 +37,9 @@ import zlib
 
 from thrift.Thrift import TApplicationException
 from thrift.protocol.TBinaryProtocol import TBinaryProtocol
-from .TTransport import TTransportException, TTransportBase, CReadableTransport
+from thrift.transport.TTransport import (
+    TTransportException, TTransportBase, CReadableTransport
+)
 from thrift.protocol.TCompactProtocol import (
     getVarint, readVarint, TCompactProtocol
 )
@@ -59,11 +59,11 @@ except ImportError:
         def decompress(self, buf):
             raise TTransportException(TTransportException.INVALID_TRANSFORM,
                                       'snappy module not available')
-    snappy = DummySnappy()  # type: ignore
+    snappy = DummySnappy()
 
 # Import the zstd module if it is available
 try:
-    import zstd
+    import zstd  # @manual
 except ImportError:
     # If zstd is not available, don't fail immediately.
     # Only raise an error if we actually ever need to perform zstd
@@ -76,7 +76,7 @@ except ImportError:
         def ZstdDecompressor(self):
             raise TTransportException(TTransportException.INVALID_TRANSFORM,
                                       'zstd module not available')
-    zstd = DummyZstd()  # type: ignore
+    zstd = DummyZstd()
 
 
 # Definitions from THeader.h
@@ -136,9 +136,11 @@ class THeaderTransport(TTransportBase, CReadableTransport):
 
     # Defaults to current user, but there is also a setter below.
     __identity = None
+    __first_request = True
     IDENTITY_HEADER = "identity"
     ID_VERSION_HEADER = "id_version"
     ID_VERSION = "1"
+    CLIENT_METADATA_HEADER = "client_metadata";
 
     def __init__(self, trans, client_types=None, client_type=None):
         self.__trans = trans
@@ -422,6 +424,9 @@ class THeaderTransport(TTransportBase, CReadableTransport):
                 self.__write_transforms.append(trans_id)
         return buf
 
+    def disable_client_metadata(self):
+        self.__first_request = False
+
     def flush(self):
         self.flushImpl(False)
 
@@ -446,6 +451,12 @@ class THeaderTransport(TTransportBase, CReadableTransport):
             self.__write_headers[self.ID_VERSION_HEADER] = self.ID_VERSION
             self.__write_headers[self.IDENTITY_HEADER] = self.__identity
 
+        if self.__first_request:
+            self.__first_request = False
+            self.__write_headers[self.CLIENT_METADATA_HEADER] = \
+                "{\"agent\":\"THeaderTransport.py\"}"
+
+
         info_data = StringIO()
 
         # Write persistent kv-headers
