@@ -1,3 +1,19 @@
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+# pyre-unsafe
+
 """
 Fuzz Testing for Thrift Services
 """
@@ -22,7 +38,8 @@ import six.moves as sm
 from six.moves.urllib.parse import urlparse
 
 try:
-    from ServiceRouter import ConnConfigs, ServiceOptions, ServiceRouter
+    # pyre-fixme[21]: Could not find module `ServiceRouter`.
+    from ServiceRouter import ConnConfigs, ServiceOptions, ServiceRouter  # @manual
     SR_AVAILABLE = True
 except ImportError:
     SR_AVAILABLE = False
@@ -30,8 +47,7 @@ except ImportError:
 from thrift import Thrift
 from thrift.transport import TTransport, TSocket, TSSLSocket, THttpClient
 from thrift.protocol import TBinaryProtocol, TCompactProtocol, THeaderProtocol
-
-from . import randomizer
+from thrift.util import randomizer
 
 if six.PY3:
     from importlib.machinery import SourceFileLoader
@@ -364,7 +380,7 @@ class FuzzerConfiguration(object):
         parent_path, service_filename = os.path.split(service_path)
         service_name = service_filename[:-3]  # Truncate extension
 
-        logging.info("Service name: %s" (service_name))
+        logging.info("Service name: %s" % (service_name))
 
         parent_path = os.path.dirname(service_path)
         ttypes_path = os.path.join(parent_path, 'ttypes.py')
