@@ -1,4 +1,18 @@
 #!/usr/bin/env python3
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 # See __init__.pyi for easier to digest types for typecheckers
 
 __all__ = []
@@ -15,13 +29,10 @@ try:
         ThriftServer,
         SSLPolicy,
         get_context,
-        pass_context,
         RequestContext,
     )
 
-    __all__.extend(
-        ["ThriftServer", "get_context", "pass_context", "SSLPolicy", "RequestContext"]
-    )
+    __all__.extend(["ThriftServer", "get_context", "SSLPolicy", "RequestContext"])
 except ModuleNotFoundError:
     pass
 
@@ -45,15 +56,15 @@ except ModuleNotFoundError:
     pass
 
 try:
-    from thrift.py3.serializer import Protocol, serialize, deserialize  # noqa: 401
+    from thrift.py3.serializer import serialize, deserialize  # noqa: 401
 
-    __all__.extend(["Protocol", "serialize", "deserialize"])
+    __all__.extend(["serialize", "deserialize"])
 except ModuleNotFoundError:
     pass
 
 try:
-    from thrift.py3.common import Priority, RpcOptions  # noqa: 401
+    from thrift.py3.common import Priority, RpcOptions, Protocol  # noqa: 401
 
-    __all__.extend(["Priority", "RpcOptions"])
+    __all__.extend(["Priority", "Protocol", "RpcOptions"])
 except ModuleNotFoundError:
     pass
