@@ -1,3 +1,17 @@
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -5,6 +19,7 @@ from __future__ import unicode_literals
 
 import unittest
 import socket
+import time
 from threading import Thread
 
 from ThriftTest import ThriftTest
@@ -33,6 +48,8 @@ def create_server():
     t.setDaemon(True)
     t.start()
 
+    time.sleep(2)
+
     return (server, port)
 
 
